//----------------------------------------------------------------------------------------------------------------
// Class definition include
//----------------------------------------------------------------------------------------------------------------

#include <string>
#include "wx/app.h"
#include "IRMMain.h"
#include "IRMMainFrame.h"

#include "creaWx.h"
#include <creaImageIOSystem.h>

#include "vtkMetaImageReader.h"
#include "vtkImageData.h"

#ifndef PATH_MAX // If not defined yet : do it 
#  define PATH_MAX 2048
#endif 

IRMMainFrame* wxTheApplication::frame = 0;

//----------------------------------------------------------------------------------------------------------------
// This macro implements the entry point (main function) for the application
//----------------------------------------------------------------------------------------------------------------

//=========================================================================  

#if defined(_WIN32)
#define CREAIRM_VALID_FILE_SEPARATOR_CHAR '\\'
#else
#define CREAIRM_VALID_FILE_SEPARATOR_CHAR '/'
#endif

//=========================================================================
int get_app_path (char *pname, size_t pathsize)
{
#ifdef LINUX	
    /* Oddly, the readlink(2) man page says no NULL is appended. */
    /* So you have to do it yourself, based on the return value: */
    pathsize --; /* Preserve a space to add the trailing NULL */
    long result = readlink("/proc/self/exe", pname, pathsize);
    if (result > 0)
	{
		pname[result] = 0; /* add the #@!%ing NULL */
		
		if ((access(pname, 0) == 0))
			return 0; /* file exists, return OK */
		/*else name doesn't seem to exist, return FAIL (falls
		 through) */
	}
#endif /* LINUX */
    
#ifdef WIN32
    long result = GetModuleFileName(NULL, pname, pathsize);
    if (result > 0)
	{
		/* fix up the dir slashes... */
		int len = strlen(pname);
		int idx;
		for (idx = 0; idx < len; idx++)
		{
			if (pname[idx] == '\\') pname[idx] = '/';
		}
		
		for (idx = len-1; idx >=0 ; idx--)
		{
			if (pname[idx] == '/')
			{ 
				pname[idx+1] = '\0';
				idx = -1;
			}
		}
		
		if ((access(pname, 0) == 0))
			return 0; /* file exists, return OK */
		/*else name doesn't seem to exist, return FAIL (falls
		 through) */
	}
#endif /* WIN32 */
    
#ifdef SOLARIS
    char *p = getexecname();
    if (p)
	{
		/* According to the Sun manpages, getexecname will
		 "normally" return an */
		/* absolute path - BUT might not... AND that IF it is not,
		 pre-pending */
		/* getcwd() will "usually" be the correct thing... Urgh!
		 */
		
		/* check pathname is absolute (begins with a / ???) */
		if (p[0] == '/') /* assume this means we have an
		 absolute path */
		{
			strncpy(pname, p, pathsize);
			if ((access(pname, 0) == 0))
				return 0; /* file exists, return OK */
		}
		else /* if not, prepend getcwd() then check if file
		 exists */
		{
			getcwd(pname, pathsize);
			long result = strlen(pname);
			strncat(pname, "/", (pathsize - result));
			result ++;
			strncat(pname, p, (pathsize - result));
			
			if ((access(pname, 0) == 0))
				return 0; /* file exists, return OK */
			/*else name doesn't seem to exist, return FAIL
			 (falls through) */
		}
	}
#endif /* SOLARIS */
    
#ifdef MACOSX /* assume this is OSX */
    /*
	 from http://www.hmug.org/man/3/NSModule.html
	 
	 extern int _NSGetExecutablePath(char *buf, unsigned long
	 *bufsize);
	 
	 _NSGetExecutablePath  copies  the  path  of the executable
	 into the buffer and returns 0 if the path was successfully
	 copied  in the provided buffer. If the buffer is not large
	 enough, -1 is returned and the  expected  buffer  size  is
	 copied  in  *bufsize.  Note that _NSGetExecutablePath will
	 return "a path" to the executable not a "real path" to the
	 executable.  That  is  the path may be a symbolic link and
	 not the real file. And with  deep  directories  the  total
	 bufsize needed could be more than MAXPATHLEN.
	 */
	
    int status = -1;
    char *given_path = (char*)malloc(MAXPATHLEN * 2);
    if (!given_path) return status;
    
    uint32_t npathsize = MAXPATHLEN * 2;
    long result = _NSGetExecutablePath(given_path, &npathsize);
    if (result == 0)
	{ /* OK, we got something - now try and resolve the real path...
	 */
		if (realpath(given_path, pname) != NULL)
		{
			if ((access(pname, 0) == 0))
				status = 0; /* file exists, return OK */
		}
	}
    free (given_path);
    return status;
#endif /* MACOSX */
    
    return -1; /* Path Lookup Failed */
} 

wxString getExecutablePath()
{
    char name[PATH_MAX];
    int err = get_app_path(name, PATH_MAX);
    if (err) 
	{
		printf(" Could not determine current executable path ?  ");  
	}
    
    // remove the exe name
    char *slash;		
    slash = strrchr(name, CREAIRM_VALID_FILE_SEPARATOR_CHAR);
    if (slash)
	{
		*slash = 0;
	}
   // desperate try to compile under Fedora
    std::string str_name(name);
    wxString wx_name = crea::std2wx(str_name);
    
    return (wx_name);
}

//=========================================================================

IMPLEMENT_APP( wxTheApplication );

bool wxTheApplication :: OnInit()
{
	wxInitAllImageHandlers();

	wxFrame* mainFrame = new wxFrame(NULL, wxID_ANY, wxT("TRAITEMENT D'IMAGES IRM multimodales D'ISCHEMIE Cerebrale - Evaluation version, 17 Juillet 2009"), wxPoint(425,50), wxSize(800, 900) );
	frame = IRMMainFrame::getInstance(mainFrame, wxID_ANY, wxString(_T("")), wxPoint(425,50), wxSize(800, 900), wxDEFAULT_FRAME_STYLE | wxSUNKEN_BORDER,(const char*)( getExecutablePath().mb_str() ) );

	mainFrame->Show(TRUE);
	return TRUE;

	/*
	wxString iconsPath = getExecutablePath()+ wxString( _T("/data/Icons") , wxConvUTF8 );
	char cstring[512];
	strcpy(cstring, (const char*)iconsPath.mb_str(wxConvUTF8));
	std::string icPath(cstring);
	*/

	/*
	size_t found;
	found=icPath.find('\\');
	while(int(found)<=icPath.size())
	{
		icPath.replace(found,1,"/");
		found=icPath.find('\\');
	}
	*/
}

#if(WIN32)
int main(int argc, char* argv[])					
  {									
    return WinMain(::GetModuleHandle(NULL), NULL,			
		   ::GetCommandLine(), SW_SHOWNORMAL);			
  }		
#else

#endif
