/*=========================================================================                                                                              
  Program:   gdcm
  Module:    $RCSfile: Test_Create.cxx,v $
  Language:  C++
  Date:      $Date: 2009/12/14 17:37:32 $
  Version:   $Revision: 1.6 $
                                                                                
  Copyright (c) CREATIS (Centre de Recherche et d'Applications en Traitement de
  l'Image). All rights reserved. See Doc/License.txt or
  http://www.creatis.insa-lyon.fr/Public/Gdcm/License.html for details.
                                                                                
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

/** 
 * 
 */
#include "IRMCreate.h"
#include "IRMCarto.h"
#include "gdcmCommon.h"
#include "gdcmDebug.h"
#include "gdcmUtil.h"
#include "gdcmDirList.h"
#include "gdcmFile.h"
#include "gdcmFileHelper.h"
#include "gdcmArgMgr.h"

#include <malloc.h>

#include <vector>
#include <stdio.h>
#include <iostream>
#include "vtkGdcmReader.h"
#include <vtkImageData.h>
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include "vtkRenderWindowInteractor.h"
#include <vtkImageViewer2.h>
#include <string>
#include <sstream>

using namespace std;

// ===================================================================================================

int main(int argc, char *argv[])
{
	START_USAGE(usage)
	" \n TestCreateCarto : \n                                            							",
	" usage: TestCreateCarto  dirin=rootDirectoryName                          						",
	"                         nb=number of Slices                                                                   (default : 1)",
	"                         res_temp=resolution temporelle                                                        (default : 1)",
	"                         nb_baseline=nombre d images necessaires pour le calcul de la baseline                 (default : 3)",
	"                         delay= choix du retard :                                                              (default : 0)",
	"						= 0 choix du retard par erreur rms,                                              ",
	"						= 1 ; choix du retard par 1-ier max sur t=0 de la courbe residue                 ",
	"						= 2 ; on prend ret=0 et on retient pour rBF la valeur du max de la courbe residue",
	"                         eps=eps sur max des valeurs singulieres 						(default :  0.1) ",
	"                         min=premiere image a prendre en compte 						(default : 8)	",
	"                         max=derniere image a prendre en compte 						(default : 100)	",
	"                         TE=temps d'echo 									(default : 0.05)",
	"                         thresnoise=seuil de bruit 								(default : 600) ",
	"                         retard_AIF=nombre de fois ou il faut decaler l entree arterielle  			(default : 1)",
	"                         retard_perf=nombre de fois ou il faut decaler la sequence de concentration  		(default : 3 ",
	"                         num_coupe=numero de la coupe par laquelle on veut commencer  				(default : 0)",
	"                         fichierAIF=nom du fichier contenant l entree arterielle initialise a AIF.csv dans repertoire bin   ",
	"                         cartoPeak=O    si on souhaite generer les cartographies du peak et ttp, =N, sinon 	(default : O)",
	"                         cartoSVD=O       si on souhaite generer les cartographies avec la SVD,  =N, sinon 	(default : N)",
	"                         cartoARMA=O      si on souhaite generer les cartographies avec ARMA,    =N, sinon 	(default : N)",
	"                         affichage=O      si on souhaite afficher les cartographies generees,    =N, sinon, 	(default : O)",
	"                         enregistrement=O si on souhaite enregistrer les cartographies generees, =N, sinon 	(default : O)",
	"                   [debug] [verbose]                                      ",
	"                                                                          ",
	"  debug      : developper wants to run the program in 'debug mode'        ",
	FINISH_USAGE

	// ------------ Initialize Arguments Manager ---------------- 

	GDCM_NAME_SPACE::ArgMgr *am= new GDCM_NAME_SPACE::ArgMgr(argc, argv);

	if (argc == 1 || am->ArgMgrDefined("usage") )
	{
		am->ArgMgrUsage(usage); // Display 'usage'
		delete am;
		return 1;
	}
// 	IRMCreate(double res_temp, int nbbaseline, int delay, double eps, int min, int max, double TE, double thresnoise, int retard_AIF, int retard_perf);
	const char *dirNamein;   
	dirNamein  =        am->ArgMgrGetString("dirin",".");

	int nb =            am->ArgMgrGetInt("nb",1);

	double res_temp =   am->ArgMgrGetFloat("res_temp",1.);
	int nbbaseline =    am->ArgMgrGetInt("nb_baseline",3);
	int delay =         am->ArgMgrGetInt("delay",0);
	double eps =        am->ArgMgrGetFloat("eps",0.1);
	int min =           am->ArgMgrGetInt("min",8);
	int max =           am->ArgMgrGetInt("max",100);
	double TE =         am->ArgMgrGetFloat("TE",0.05);
	double thresnoise = am->ArgMgrGetInt("thresnoise",600);
	int retard_AIF =    am->ArgMgrGetInt("retard_AIF",1);
	int retard_perf =   am->ArgMgrGetInt("retard_perf",3);
	int numcoupe =      am->ArgMgrGetInt("num_coupe",0);

	const char *fic;
	fic  = am->ArgMgrGetString("fichierAIF","AIF.csv");

	const char *cartoMax;
	cartoMax=am->ArgMgrGetString("cartoPeak","O");

	const char *cartoSVD;
	cartoSVD=am->ArgMgrGetString("cartoSVD","O");

	const char *cartoARMA;
	cartoARMA=am->ArgMgrGetString("cartoARMA","N");

	const char *aff;
	aff=am->ArgMgrGetString("affichage","O");

	const char *enreg;
	enreg=am->ArgMgrGetString("enregistrement","O");

	if (am->ArgMgrDefined("debug"))
		  GDCM_NAME_SPACE::Debug::DebugOn();

	bool verbose  = am->ArgMgrDefined("verbose");      

	/* if unused Param we give up */
	if ( am->ArgMgrPrintUnusedLabels() )
	{
		am->ArgMgrUsage(usage);
		delete am;
		return 1;
	}

	delete am;  // we don't need Argument Manager any longer  

	// ----------- End Arguments Manager ---------

	 //2dseq_Slice_0.dcm   

	// ----- Begin Processing -----

	std::vector <vtkImageData *> output;
	char nomFich[1024]; // Hope it's enough !

	IRMCreate *create = new IRMCreate(res_temp, nbbaseline, delay, eps, min, max, TE, thresnoise, retard_AIF, retard_perf);
	vnl_matrix<double> entree_arterielle=(create->getCarto())->lectureAIFdansexcel(fic);

	int i;
	// Declare usefull objects;

 	vtkImageData *image_max =      0;
	vtkImageData *image_ttp =      0;
	vtkImageData *image_mtt_svd =  0;
	vtkImageData *image_cbv_svd =  0;
	vtkImageData *image_cbf_svd =  0;
	vtkImageData *image_tmax_svd = 0;
	vtkImageData *image_mtt_arma = 0;
	vtkImageData *image_cbv_arma = 0;
	vtkImageData *image_cbf_arma = 0;

	vtkGdcmReader *reader = vtkGdcmReader::New();

	for (i = 0; i < nb; i++)
	{
		if (verbose)
			#if defined(_WIN32)
				system("date /T");
			#else
				system("date");
			#endif

		//reader->SetFlipY(false);

		sprintf(nomFich, "%s%c2dseq_Slice_%d.dcm", dirNamein, GDCM_NAME_SPACE::GDCM_FILESEPARATOR, numcoupe);
		if (verbose)
			std::cout << "file to be processed : [" << nomFich << "]" << std::endl;
		reader->SetFileName(nomFich);
		reader->Update();
		output.push_back(reader->GetOutput());

		create->addImage(output.at(i));
		create->MoyennageImage(i);
		create->CalculSeqCon(i);

		if(strcmp(cartoMax,"O") == 0)
		{
		        /// \TODO : delete the objects, later
			/*if (!image_max) */ image_max = vtkImageData::New();
			/*if (!image_ttp) */ image_ttp = vtkImageData::New();
		
			//creation cartographies max et ttpmax
			std::cout << " creation cartographies max et ttpmax" << std::endl;
			create->createCartoMAX_TTP(i, image_max, image_ttp);
			std::cout << " OK " <<std::endl;
		}

		if(strcmp(cartoSVD,"O") == 0)
		{
		        /// \TODO : delete the objects, later
			/*if (!image_mtt_svd)  */  image_mtt_svd  =  vtkImageData::New();
			/*if (!image_cbv_svd)  */  image_cbv_svd  =  vtkImageData::New();
			/*if (!image_cbf_svd)  */  image_cbf_svd  =  vtkImageData::New();
			/*if (!image_tmax_svd) */  image_tmax_svd =  vtkImageData::New();
			//creation cartographies de la svd
			std::cout << " creation cartographies de la svd " << std::endl;
			create->createCartoSVD(i, entree_arterielle, image_mtt_svd, image_cbv_svd, image_cbf_svd, image_tmax_svd);
		}

		if(strcmp(cartoARMA,"O") == 0)
		{
		        /// \TODO : delete the objects, later
			/*if (!image_mtt_arma)  */ image_mtt_arma = vtkImageData::New();
			/*if (!image_cbv_arma)  */ image_cbv_arma = vtkImageData::New();
			/*if (!image_cbf_arma)  */ image_cbf_arma = vtkImageData::New();
			//creation cartographies avec ARMA
			std::cout << " creation cartographies ARMA " << std::endl;
			create->createCartoARMA(i, entree_arterielle, image_mtt_arma, image_cbv_arma, image_cbf_arma);
			std::cout << " OK " <<std::endl;
		}

		if(strcmp(aff,"O")==0)
		{
		      std::cout << " 'Affichage' was requiered " << std::endl;
		      create->affichageImage(create->getSelectedImage(i));
		      create->affichageImage(create->getSelectedImageMoy(i));
		      create->affichageImage(create->getSelectedSeqCon(i));

		      if(strcmp(cartoMax,"O") == 0)
		      {
			      create->affichageImage(create->getVectMAX().at(i));
			      create->affichageImage(create->getVectTTP().at(i));
		      }

		      if(strcmp(cartoSVD,"O") == 0)
		      {
			      create->affichageImage((create->getVectMTTSVD()).at(i));
			      create->affichageImage((create->getVectCBVSVD()).at(i));
			      create->affichageImage((create->getVectCBFSVD()).at(i));
			      create->affichageImage((create->getVectTMAXSVD()).at(i));
		      }

		      if(strcmp(cartoARMA,"O") == 0)
		      {
			      create->affichageImage((create->getVectMTTARMA()).at(i));
			      create->affichageImage((create->getVectCBVARMA()).at(i));
			      create->affichageImage((create->getVectCBFARMA()).at(i));
		      }
		}
		numcoupe++;
	}

	if(strcmp(enreg,"O")==0)
	{
		std::cout << " 'enreg' was requiered " << std::endl;
		if(strcmp(cartoMax,"O")==0)
		{
			char *nom_max=(char*)malloc(4*sizeof(char));
			char *nom_ttp=(char*)malloc(4*sizeof(char));
			sprintf(nom_max, "MAX.mhd");
			create->enregistrercarto(create->getVectMAX(), nom_max, nb);
			sprintf(nom_ttp, "TTP.mhd");
			create->enregistrercarto(create->getVectTTP(), nom_ttp, nb);
		}

		if(strcmp(cartoSVD,"O")==0)
		{
			char *nom_mttsvd=(char*)malloc(7*sizeof(char));
			char *nom_cbvsvd=(char*)malloc(7*sizeof(char));
			char *nom_cbfsvd=(char*)malloc(7*sizeof(char));
			char *nom_tmaxsvd=(char*)malloc(8*sizeof(char));
			sprintf(nom_mttsvd, "MTTsvd.mhd");
			create->enregistrercarto(create->getVectMTTSVD(), nom_mttsvd, nb);
			sprintf(nom_cbvsvd, "CBVsvd.mhd");
			create->enregistrercarto(create->getVectCBVSVD(), nom_cbvsvd, nb);
			sprintf(nom_cbfsvd, "CBFsvd.mhd");
			create->enregistrercarto(create->getVectCBFSVD(), nom_cbfsvd, nb);
			sprintf(nom_tmaxsvd, "TMAXsvd.mhd");
			create->enregistrercarto(create->getVectTMAXSVD(), nom_tmaxsvd, nb);
		}

		if(strcmp(cartoARMA,"O")==0)
		{
			char *nom_mttarma=(char*)malloc(7*sizeof(char));
			char *nom_cbvarma=(char*)malloc(7*sizeof(char));
			char *nom_cbfarma=(char*)malloc(7*sizeof(char));
			sprintf(nom_mttarma, "MTTarma.mhd");
			create->enregistrercarto(create->getVectMTTARMA(), nom_mttarma, nb);
			sprintf(nom_cbvarma, "CBVarma.mhd");
			create->enregistrercarto(create->getVectCBVARMA(), nom_cbvarma, nb);
			sprintf(nom_cbfarma, "CBFarma.mhd");
			create->enregistrercarto(create->getVectCBFARMA(), nom_cbfarma, nb);
		}
	}

	// Delete all objects before leaving!
    /// \TODO : delete *all* of them, not only the last one!  // JPR
 	if (image_max)      image_max->Delete();
	if (image_ttp)      image_ttp->Delete();
	if (image_mtt_svd)  image_mtt_svd->Delete();
	if (image_cbv_svd)  image_cbv_svd->Delete();
	if (image_cbf_svd)  image_cbf_svd->Delete();
	if (image_tmax_svd) image_tmax_svd->Delete();
	if (image_mtt_arma) image_mtt_arma->Delete();
	if (image_cbv_arma) image_cbv_arma->Delete();
	if (image_cbf_arma) image_cbf_arma->Delete();

	reader->Delete();

	return 0;
}


