//----------------------------------------------------------------------------------------------------------------
// Class definition include
//----------------------------------------------------------------------------------------------------------------
#include "IRMCarto.h"

using namespace std;

/*************************************************************************************************************/
/*Constructors & Destructors**********************************************************************************/
/*************************************************************************************************************/

IRMCarto::IRMCarto(double res_temp,int nbbaseline, int delay, double eps, int min, int max, double TE, double thresnoise, int retard_AIF, int retard_perf)
{

std::cout << "Constructor IRMCarto : res_temp " << res_temp << " nbbaseline " << nbbaseline << " delay " << delay << " eps " << eps
		<< " min " << min << " max " << max << " TE " << TE << " thresnoise " << thresnoise 
		<< " retard_AIF " << retard_AIF << " retard_perf " << retard_perf << std::endl;

	_res_temp    = res_temp;
	_nbbaseline  = nbbaseline;
	_delay       = delay;
	_eps         = eps;
	_min         = min;
	_max         = max;
	_TE          = TE;
	_thresnoise  = thresnoise;	
	_retard_AIF  = retard_AIF;
	_retard_perf = retard_perf;
}

/************************************************************************************/
IRMCarto::~IRMCarto()
{
	printf("EED IRMCarto::~IRMCarto\n");
}	


/*************************************************************************************************************/
/*Ajout, suppression, selection d'images**********************************************************************/
/*************************************************************************************************************/

std::vector<vtkImageData*> IRMCarto::getVectImages()
{
	return _vectimages;
}

/*Methode qui ajoute la vtkImageData en derniere position du vecteur*******************/
void IRMCarto::addImage(vtkImageData * img)
{
	_vectimages.push_back(img);
}

/*Methode qui supprime la vtkImageData a la position selected*************************/
void IRMCarto::removeImage(int selected)
{
	_vectimages.erase (_vectimages.begin()+(selected));
}

/*Methode qui retourne la vtkImageData dont la position est pos**********************/
vtkImageData* IRMCarto::getSelectedImage(int pos)
{
//std::cout << "in IRMCarto::getSelectedImage taille _vectimages " <<_vectimages.size() << std::endl; 
//std::cout << "in IRMCarto::getSelectedImage : select im " << pos << std::endl;
//_vectimages[pos]->PrintSelf(std::cout, vtkIndent(2));
	return _vectimages[pos];
}

/*************************************************************************************************************/
/*Ajout, suppression, selection d'images moyennees************************************************************/
/*************************************************************************************************************/
std::vector<vtkImageData*> IRMCarto::getVectImagesMoy()
{
	return _vectimagesMoy;
}

void IRMCarto::addImageMoy(vtkImageData * img)
{
	_vectimagesMoy.push_back(img);
}
	
void IRMCarto::removeImageMoy(int selected)
{
	_vectimagesMoy.erase (_vectimages.begin()+(selected));
}

vtkImageData* IRMCarto::getSelectedImageMoy(int pos)
{
	return _vectimagesMoy[pos];
}
/*************************************************************************************************************/
/*Ajout, suppression, selection des sequences de concentration************************************************/
/*************************************************************************************************************/
std::vector<vtkImageData*> IRMCarto::getVectSeqCon()
{
	return _vectseqcon;
}

/*Methode qui ajoute la vtkImageData en derniere position du vecteur*******************/
void IRMCarto::addSeqCon(vtkImageData * img)
{
	_vectseqcon.push_back(img);
}

/*Methode qui supprime la vtkImageData a la position selected*************************/
void IRMCarto::removeSeqCon(int selected)
{
	_vectseqcon.erase (_vectseqcon.begin()+(selected));
}

/*Methode qui retourne la vtkImageData dont la position est pos**********************/
vtkImageData* IRMCarto::getSelectedSeqCon(int pos)
{
	return _vectseqcon[pos];
}

/*************************************************************************************************************/
/*Creation d'un pointeur sur une vtkImageData*****************************************************************/
/*************************************************************************************************************/
double * IRMCarto::pointageImageDouble(vtkImageData *img, int index_temps, int nb_lignes, int nb_colonnes)
{
	int indice=index_temps*(nb_lignes*nb_colonnes);
	return ((double*)(img->GetScalarPointer())+indice);
}

/*************************************************************************************************************/
/*Moyennage d'image*******************************************************************************************/
/*************************************************************************************************************/
/*Methode qui moyenne l'image**********************************************************************************/
void IRMCarto::moyennage3x3(vtkImageData *img,vnl_matrix<double> filtre)
{
	setDim(img);
	int sizeX=getDim(1);
	int sizeY=getDim(0);
	int sizeZ=getDim(2);
	int i,j,index_temps;
	int index_lignes=0;
	int index_colonnes=0;
	double valeur;

	int filtre_nbcolonnes=(int)filtre.cols();
	int filtre_nblignes=(int)filtre.rows();

	if(sizeY!=sizeX || filtre_nbcolonnes!=filtre_nblignes || filtre_nbcolonnes!=3)
	{
		perror("Erreur: matrices non carrees");
		exit(1);
	}
	
	int milieu=filtre_nbcolonnes-1;
	
	if(milieu%2==0)
	{
		milieu=milieu/2;
	}
	else
	{
		perror("Erreur: filtre de dimensions paires");
		exit(1);
	}

	vtkImageData *resultat=vtkImageData::New();
	resultat->SetDimensions(sizeY,sizeX,sizeZ);
	resultat->SetScalarTypeToDouble();
	resultat->SetExtent(0, sizeY-1, 0, sizeX-1, 0, sizeZ-1);
	resultat->SetNumberOfScalarComponents(1);
	resultat->AllocateScalars();

	for(index_temps=0; index_temps<sizeZ; index_temps++)
	{
		for(index_lignes=0; index_lignes<sizeX; index_lignes++)
		{
			for(index_colonnes=0; index_colonnes<sizeY; index_colonnes++)
			{
				for(i=-1; i<=1; i++)
				  {
					  for(j=-1; j<=1; j++)
					  {
						  if(index_lignes-i>=0 && index_lignes-i<sizeX && index_colonnes-j>=0 && index_colonnes-j<sizeY)
						  {
							  valeur=resultat->GetScalarComponentAsDouble(index_colonnes,index_lignes,index_temps,0)+img->GetScalarComponentAsDouble(index_colonnes-j,index_lignes-i,index_temps,0)*filtre[i+1][j+1];
							  resultat->SetScalarComponentFromDouble(index_colonnes,index_lignes,index_temps,0,valeur);
						    }
					    }     
				  }
			}
		}
	}
	addImageMoy(resultat);
}

/*************************************************************************************************************/
/*Operations sur les matrices*********************************************************************************/
/*************************************************************************************************************/
/*Methode qui transforme un vecteur ligne en un vecteur colonne***********************************************/
vnl_matrix<double> IRMCarto::vectligne2col(vnl_matrix<double> vectlignes )
{
	int sizeX=(int)vectlignes.rows();
	int sizeY=(int)vectlignes.cols();
	vnl_matrix<double> vectcol(sizeY,sizeX);
	for(int index_colonnes=0;index_colonnes<sizeY;index_colonnes++)
	{
		vectcol.put(index_colonnes,0,vectlignes.get(0,index_colonnes));
	}
	return vectcol;
}

/*Methode qui retourne la valeur du vecteur vect a la position pos*********************************************/
void IRMCarto::maxvecteur(vnl_matrix<double> vect,double *maxValue,int *posValue)
{
	int sizeX=(int)vect.rows();
	int sizeY=(int)vect.cols();
	int i,j;

	//initialisation au premier element 
	*posValue=0;
	*maxValue=vect.get(0,0);

	for(i=0; i<sizeX;i++)
	{
		for(j=0; j<sizeY;j++)
		{
			if(vect.get(i,j)>*maxValue)
			{
				*maxValue=vect.get(i,j);

				if(sizeX==1)
					*posValue=j;

				if(sizeY==1)
					*posValue=i;
			}
		}
	}
}

/*Methode qui realise un filtrage (fonction filter sous matlab)**************************************************/
vnl_matrix<double> IRMCarto::filter(vnl_matrix<double> b, vnl_matrix<double> a, vnl_matrix<double> x)
{
	int x_nblignes   = (int)x.rows();
	int x_nbcolonnes = (int)x.cols();
	int a_nblignes   = (int)a.rows();
	int a_nbcolonnes = (int)a.cols();
	int b_nblignes   = (int)b.rows();
	int b_nbcolonnes = (int)b.cols();

	vnl_matrix<double> resultat(x_nblignes, x_nbcolonnes,0);

	int i,j;
	int sous;

	if(x_nbcolonnes==1)
	{
		for(i=0; i<x_nblignes; i++)
		{
			if(b_nbcolonnes==1)
			{
				for(j=0; j<b_nblignes;j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[i][0]=resultat[i][0]+(b[j][0]*x[sous][0]);
					}

					else
					{
						j=b_nblignes;
					}
				}
			}

			else if(b_nblignes==1)
			{
				for(j=0; j<b_nbcolonnes;j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[i][0]=resultat[i][0]+(b[0][j]*x[sous][0]);
					}
					
					else
					{
						j=b_nbcolonnes;
					}
				}
			}

			if(a_nbcolonnes==1)
			{
				for(j=1; j<a_nblignes; j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[i][0]=resultat[i][0]-(a[j][0]*resultat[sous][0]);
					}

					else
					{
						j=a_nblignes;
					}
				}
			}

			else if(a_nblignes==1)
			{
				for(j=1; j<a_nbcolonnes; j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[i][0]=resultat[i][0]-(a[0][j]*resultat[sous][0]);
					}
					
					else
					{
						j=a_nbcolonnes;
					}
				}
			}
		}
	}

	else if(x_nblignes==1)
	{
		for(i=0; i<x_nbcolonnes; i++)
		{
			if(b_nbcolonnes==1)
			{
				for(j=0; j<b_nblignes;j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[0][i]=resultat[0][i]+(b[j][0]*x[0][sous]);
					}
					
					else
					{
						j=b_nblignes;
					}
				}
			}

			if(b_nblignes==1)
			{
				for(j=0; j<b_nbcolonnes;j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[0][i]=resultat[0][i]+(b[0][j]*x[0][sous]);
					}
					
					else
					{
						j=b_nbcolonnes;
					}
				}
			}

			if(a_nbcolonnes==1)
			{
				for(j=1; j<a_nblignes; j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[0][i]=resultat[0][i]-(a[j][0]*resultat[0][sous]);
					}
					
					else
					{
						j=a_nblignes;
					}
				}
			}

			if(a_nblignes==1)
			{
				for(j=1; j<a_nbcolonnes; j++)
				{
					sous=i-j;
					if(sous>=0)
					{
						resultat[0][i]=resultat[0][i]-(a[0][j]*resultat[0][sous]);
					}
					
					else
					{
						j=a_nbcolonnes;
					}
				}
			}
		}
	}
	return resultat;
}

/*************************************************************************************************************/
/*Accesseurs et mutateurs de la classe************************************************************************/
/*************************************************************************************************************/

/************************************************************************************/
void IRMCarto::setDim(vtkImageData* img)
{
	_dimEntree=img->GetDimensions();
}

/************************************************************************************/
int IRMCarto::getDim(int n)
{
	if(n<0 || n>2)
	{
		return -1;
	}
	else
	{
		return (_dimEntree[n]);
	}
}

/************************************************************************************/
void IRMCarto::setDimMoy(vtkImageData* img)
{
	_dimMoy=img->GetDimensions();
}

/************************************************************************************/
int IRMCarto::getDimMoy(int n)
{
	if(n<0 || n>2)
	{
		return -1;
	}
	else
	{
		return (_dimMoy[n]);
	}
}

/************************************************************************************/
void IRMCarto::setDimSeqCon(vtkImageData* img)
{
	_dimSeqCon=img->GetDimensions();
}

/************************************************************************************/
int IRMCarto::getDimSeqCon(int n)
{
	if(n<0 || n>2)
	{
		return -1;
	}
	else
	{
		return (_dimSeqCon[n]);
	}
}

/*Methode qui cree le vecteur concentration a partir de la matrice image**************/
void IRMCarto::setVectConcentration(vtkImageData* img,int index_colonnes, int index_lignes)
{	
	int index_temps;
	int sizeX=getDimSeqCon(1);
	int sizeY=getDimSeqCon(0);
	int sizeZ=getDimSeqCon(2);

	_concentration.set_size(1,sizeZ);
	double *p;

	for(index_temps=0; index_temps<sizeZ;index_temps++)
	{
		p =pointageImageDouble(img,index_temps,sizeX,sizeY);
		p=p+index_lignes * sizeX + index_colonnes;
		_concentration[0][index_temps]= *p ;			
	}
}

/*******************************************************************************************/
vnl_matrix<double> IRMCarto::getVectConcentration()
{
	return _concentration;
}

/*******************************************************************************************/
double IRMCarto::getconcentration(int pos)
{
	return _concentration.get(0,pos);;
}

/*******************************************************************************************/
double IRMCarto:: getmtt()
{
	return mtt;
}

/*******************************************************************************************/
double IRMCarto:: getcbv()
{
	return cbv;
}

/*******************************************************************************************/
double IRMCarto:: getcbf()
{
	return cbf;
}

/*******************************************************************************************/
int IRMCarto:: gettmax()
{
	return tmax;
}

/*******************************************************************************************/
double IRMCarto:: getrestemp()
{
	return _res_temp;
}

/*******************************************************************************************/
int IRMCarto:: getmax()
{
	return _max;
}

/*******************************************************************************************/
int IRMCarto:: getmin()
{
	return _min;
}

/*******************************************************************************************/
vnl_matrix<double> IRMCarto::getvertis()
{
	return vertis;
}

/*************************************************************************************************************/
/*Calcul de la sequence de concentration a partir d'une vtkImageData*****************************************/
/*************************************************************************************************************/

/*Methode qui cree la matrice baseline_map********************************************************************/
vnl_matrix<double> IRMCarto::construct_baseline(vtkImageData *img)
{
	setDimMoy(img);		//construction a partir des images moyennees

	int index_lignes, index_colonnes,index_temps;
	int sizeX=getDimMoy(1);		//dimension des lignes
	int sizeY=getDimMoy(0);		//dimensions des colonnes
	int sizeZ=_nbbaseline +_min;

	vnl_matrix<double> baseline_map(sizeX,sizeY,0.);		//initialisation d'une matrice avec des 0

	for(index_temps=_min; index_temps<sizeZ; index_temps++)
	{
		for(index_lignes=0; index_lignes<sizeX; index_lignes++)
		{
			for(index_colonnes=0; index_colonnes<sizeY; index_colonnes++)
			{
				baseline_map[index_lignes][index_colonnes] = baseline_map[index_lignes][index_colonnes] + img->GetScalarComponentAsDouble(index_colonnes,index_lignes,index_temps,0);
			}
		}
	}

	baseline_map=baseline_map/_nbbaseline;

	return baseline_map;
}

/*Methode qui cree la sequence de concentration****************************************************************/
void IRMCarto::construct_seqcon(vtkImageData *img)
{
	setDimMoy(img);
	int sizeX=getDimMoy(1);
	int sizeY=getDimMoy(0);
	int sizeZ=_max-_min;

	double valeur,result,result1, result2;
	vnl_matrix<double> baseline_map=construct_baseline(img);

	//creation de l'image des la sequence de concentration
	vtkImageData *seqcon=vtkImageData::New();
	seqcon->SetDimensions(sizeY,sizeX,sizeZ);
	seqcon->SetScalarTypeToDouble();
	seqcon->SetExtent(0, sizeY-1, 0, sizeX-1, 0, sizeZ-1);
	seqcon->SetNumberOfScalarComponents(1);
	seqcon->AllocateScalars();

	int index_lignes, index_colonnes,index_temps;

	for(index_lignes=0; index_lignes<sizeX; index_lignes++)
	{
		for(index_colonnes=0; index_colonnes<sizeY; index_colonnes++)
		{
			if(baseline_map[index_lignes][index_colonnes]>_thresnoise)
			{
				for(index_temps=0; index_temps<sizeZ; index_temps++)
				{
					valeur=img->GetScalarComponentAsDouble(index_colonnes,index_lignes,index_temps+_min,0);
					if(valeur!=0)
					{
						result=baseline_map[index_lignes][index_colonnes]/valeur;
						result1=log(result);
						if(isinf(result1) || isnan(result1))
						{
							result2=0; 
						}
						else
						{
							result2=1/_TE*result1;
							//result3=-result2;
						}
						seqcon->SetScalarComponentFromDouble(index_colonnes,index_lignes,index_temps,0, result2);
					}
					else
					{
						seqcon->SetScalarComponentFromDouble(index_colonnes,index_lignes,index_temps,0, 0);
					}
				}
			}
			else
			{
				for(index_temps=0; index_temps<sizeZ; index_temps++)
				{
					seqcon->SetScalarComponentFromDouble(index_colonnes,index_lignes,index_temps,0, 0);	
				}
			}
		}
	}
	//ajout au vecteur de sequence de concentration
	addSeqCon(seqcon);	
}

/*************************************************************************************************************/
/*Calcul de la SVD et des elements de la classe***************************************************************/
/*************************************************************************************************************/

/*Methode qui recupere l'AIF a partir d'un fichier .cvs************************************/
vnl_matrix<double> IRMCarto::lectureAIFdansexcel(const char* nom_fic)
{
std::cout << " in IRMCarto::lectureAIFdansexcel ?!?" << std::endl;

	FILE* fichier = NULL;
	fichier = fopen(nom_fic, "r+");

	vnl_matrix<double> entree;

	if (fichier != NULL)
	{
		int caractereActuel = 0;
		int count=1;

		//determination du nombre de colonnes de AIF
		do
		{
			caractereActuel = fgetc(fichier); // On lit le caractere
			if(caractereActuel==' ')	//on compte les espaces car deux elements sont separes par un espace
			{
				count++;
			}
    
		} while(caractereActuel != EOF);	// On continue tant que fgetc n'a pas retourne EOF (fin de fichier)

		entree.set_size(count,1);

		fseek (fichier,0, SEEK_SET); 	//retour au debut du fichier

		for(int index_lignes=0;index_lignes<count;index_lignes++)
		{
			fscanf(fichier,"%lf",&entree[index_lignes][0]); 	
		}
	        fclose(fichier);		
	}

	else
	{
	        // Pour arriver ici, on aura verifie que le fichier existe !
		std::cout << "Impossible d'ouvrir le fichier AIF.csv -et on s'en tape un peu?-" << std::endl;
		//perror("Impossible d'ouvrir le fichier AIF.csv"); // JPR
		// exit(1);
	}
	return entree;
}

/*Methode qui calcule la SVD*********************************************************/
vnl_matrix<double> IRMCarto::calculSVD(vnl_matrix<double> perf,vnl_matrix<double> entree_arterielle)
{
	/*****************************************************************************************/
	/*******************************declaration des variables*********************************/
	/*****************************************************************************************/
	int index_lignes,index_colonnes;
	int retard;

	double ecctis=1e36;
	int posmtis=100;
	double eccS, veps, maxValue,sauve;
	int posValue,min_len,min_len2;

	vnl_matrix<double> entree=entree_arterielle;
	vnl_matrix<double> perf1;
	vnl_matrix<double> entree1;
	vnl_matrix<double> perfexc;
	vnl_matrix<double> A;
	vnl_matrix<double> U;
	vnl_matrix<double> V;
	vnl_diag_matrix<vnl_svd<double>::singval_t > W;
	vnl_matrix<double> bi;
	vnl_matrix<double> bitis;

	int perf_nbcolonnes   = (int)perf.cols();
	int entree_nbcolonnes = (int)entree.cols();
	int perf_nblignes     = (int)perf.rows();
	int entree_nblignes   = (int)entree.rows();
	int entree_size       = (int)entree.size();
	int perf_size         = (int)perf.size();

	int entree1_size, perf1_size,bi_nblignes;

	/*****************************************************************************************/
	/******verification que entree et perf sont des vecteurs colonnes*************************/
	/*****************************************************************************************/
	if((entree_nbcolonnes!=1)||(perf_nbcolonnes!=1)) 
	{
		std::cout << "Erreur (1) dans le vecteur entrees dans Arx : entree_nbcolonnes " 
			   << entree_nbcolonnes << " perf_nbcolonnes " << perf_nbcolonnes << std::endl;
		exit(1);
	}

	/******************************************************************************************/
	/*******************suppression d'un retard eventuel sur la courbe d'entree****************/
	/*****************************************************************************************/

	if(_delay!=2)
	{
		//copie du vecteur entree dans entree1
		entree1.set_size(entree_nblignes,entree_nbcolonnes);
		entree1=entree;
		entree1_size=entree.size();

		//gettimeofday(&td,NULL);
		for(retard=_retard_AIF; retard>0; retard--)
		{  
			//on retarde l'entree
			entree=entree1.extract(entree1_size-retard,1, retard,0);
		 	entree_size=(int)entree.size();

			// on prend un nb de donnees egales a min(length(x),length(y))
			min_len=std::min(perf_size,entree_size);

			//Construction de la matrice A
			A.set_size(min_len,min_len);
			A.fill(0);
			for(index_lignes=0;index_lignes<min_len;index_lignes++)
			{
				for(index_colonnes=0;index_colonnes<index_lignes+1;index_colonnes++)
				{
					A.put(index_lignes,index_colonnes,entree.get(index_lignes-index_colonnes,0));
				}
			}

			//Calcul de TETHA=((At.A)^-1).At.B
			vnl_svd<double> mat_svd(A, 1e-8);
			U=mat_svd.U();
			V=mat_svd.V();
			W=mat_svd.W();

			// on inverse les valeurs propres superieures a max*_eps
			veps=W[0]*_eps;
			for (index_lignes=0;index_lignes<min_len;index_lignes++)
			{
				if (W[index_lignes]>veps)
				{
					W[index_lignes]=1/W[index_lignes];
				}
				      
				else
				{
					W[index_lignes]=0;
				}
			}

			//definition de la matrice residue et de eccS  
			perfexc=perf.extract(min_len,1,0,0);
			bi=V*W*(U.transpose())*perfexc;
			vertis=A*bi;
			sauve=(vertis-perfexc).frobenius_norm ();
			eccS=sauve*sauve/min_len;

			//calcul du maximum
			maxvecteur(bi,&maxValue,&posValue);

			if(_delay==0)
			{
				if(eccS<ecctis)
				{
					ecctis=eccS;
					bitis=bi;
				}
  
				else if (posValue<posmtis)
				{
					ecctis=eccS;
					bitis=bi;
					posmtis=posValue;
				}
			}
		}//fin de for
	}//fin du if 

	/*****************************************************************************************/
	/***************suppression d'un retard eventuel sur la courbe de perfusion****************/
	/*****************************************************************************************/	
	//copie du vecteur entree dans entree1
	perf1.set_size(perf_nblignes,perf_nbcolonnes);
	perf1=perf;
	perf1_size=perf_size;

	for(retard=0; retard<_retard_perf; retard++)
	{
		//on retarde la perfusion
		perf=perf1.extract(perf1_size-retard,1,retard,0);
		perf_size=(int)perf.size();	

		// on prend un nb de donnees egales a min(length(x),length(y))
		min_len2=std::min(perf_size,entree_size);

		//si on est different du cas precedent, on fait la deconvolution
		if(_delay==2 || min_len!=min_len2)
		{
		      //Construction de la matrice A
		      A.set_size(min_len2,min_len2);
		      A.fill(0);
		      for(index_lignes=0;index_lignes<min_len2;index_lignes++)
		      {
			      for(index_colonnes=0;index_colonnes<index_lignes+1;index_colonnes++)
			      {
				      A.put(index_lignes,index_colonnes,entree.get(index_lignes-index_colonnes,0));
			      }
		      }

		      //Calcul de TETHA=((At.A)^-1).At.B
		      vnl_svd<double> mat_svd(A, 1e-8);
		      U=mat_svd.U();
		      V=mat_svd.V();
		      W=mat_svd.W();

		      // on inverse les valeurs propres superieures a max*_eps
		      veps=W[0]*_eps;
		      for (index_lignes=0;index_lignes<min_len2;index_lignes++)
		      {
			      if (W[index_lignes]>veps)
			      {
				      W[index_lignes]=1/W[index_lignes];
			      }
	  
			      else
			      {
				      W[index_lignes]=0;
			      }
		      }
		}

		//definition de la matrice residue et de eccS
		bi=V*W*(U.transpose())*(perf.extract(min_len2,1,0,0));
		vertis=A*bi;
		sauve=(vertis-(perf1.extract(min_len2,1,retard,0))).frobenius_norm ();
		eccS=sauve*sauve/min_len2;

		maxvecteur(bi,&maxValue,&posValue);

		if(_delay==0)
		{
			 if(eccS<ecctis)
			{
				ecctis=eccS;
				bitis=bi;		
			}
			    
			else if(posValue<posmtis)
			{
				ecctis=eccS;
				bitis=bi;
				posmtis=posValue;
			 }
		}   

		else if(_delay==2)
		{
			ecctis=eccS;
			bi_nblignes=(int)bi.rows();
			bitis=bi.extract(bi_nblignes-posValue,1,posValue,0);					
		}

	}//fin du for
	
	if(_delay==2 || _delay==0)
	{
	    return bitis;
	}
	
	else
	{
	    return bi;
	}
}

/*Calcul du MTT sur la fonction residue**********************************************/
void IRMCarto::mttr_tmax(vnl_matrix<double> r)
{ 	
	double maxValue;
	int posValue;
	maxvecteur(r,&maxValue,&posValue);
	cbf=maxValue;
	tmax= posValue;
	int i;
	double var_m=0;

	int lon_res=(int)r.size();

	if(lon_res<2)
	{
		mtt=0.01;
		cbv=0;
	}

	else
	{
	      for(i=tmax; i<lon_res;i++)
	      {
		      var_m=var_m+r.get(i,0);
	      }

	      var_m=var_m*_res_temp;

	      if(cbf!=0)
	      {
		      mtt=var_m/cbf;
	      }

	      else
	      {
		      mtt=0;
	      }

	      cbf=cbf/_res_temp;
	      cbv=cbf*mtt;
	}
}

/*************************************************************************************************************/
/*Calcul de ARMA et des elements de la classe*****************************************************************/
/*************************************************************************************************************/

/**********************************************************************************************************/
void IRMCarto::calculARMA(vnl_matrix<double> perf,vnl_matrix<double> entree, int na, int nb)
{
	/*****************************************************************************************/
	/*******************************declaration des variables*********************************/
	/*****************************************************************************************/
	int index_lignes;
	int retard,min_len;
	double eccS,sauve;
	double ecctis=1e80;
	int rettis;

	vnl_matrix<double> perf1=perf;
	vnl_matrix<double> entree1=entree;
	vnl_matrix<double> Z;
	vnl_matrix<double> bi;
	vnl_matrix<double> bitis;
	vnl_matrix<double> verS;
	vnl_matrix<double> X;
	vnl_matrix<double> vertis;
	vnl_matrix<double> perftis;
	
	int perf_nbcolonnes   = (int)perf.cols();
	int entree_nbcolonnes = (int)entree.cols();
	int perf_nblignes     = (int)perf.rows();
	int entree_size       = (int)entree.size();
	int perf_size         = (int)perf.size();
	int entree1_size      = entree_size;
	int perf1_size        = perf_size;
	int perf1_nblignes    = perf_nblignes;
	int perf1_nbcolonnes  = perf_nbcolonnes;

	int Z_nblignes, Z_nbcolonnes;

	/*****************************************************************************************/
	/******verification que entree et perf sont des vecteurs colonnes*************************/
	/*****************************************************************************************/
	if((entree_nbcolonnes!=1)||(perf_nbcolonnes!=1)) 
	{
		std::cout << "Erreur (2) dans le vecteur entrees dans Arx : entree_nbcolonnes " 
			   << entree_nbcolonnes << " perf_nbcolonnes " << perf_nbcolonnes << std::endl;
		exit(1);
	}

	/*****************************************************************************************/
	/***************suppression d'un retard eventuel sur la courbe de perfusion****************/
	/*****************************************************************************************/	
	 
	//gettimeofday(&td,NULL);
	
	for(retard=0; retard<_retard_perf; retard++)
	{
		//on retarde la perfusion
		perf=perf1.extract(perf1_size-retard,1,retard,0)-perf1.get(retard,0);
		perf_size=(int)perf.size();	

		// on prend un nb de donnees egales a min(length(x),length(y))
		min_len=std::min(perf_size,entree_size);

		perf=perf.extract(min_len,1,0,0);
		entree=entree1.extract(min_len,1,0,0);

		//construction de la matrice Z
		Z_nblignes=min_len;
		Z_nbcolonnes=2;
		Z.set_size(Z_nblignes,Z_nbcolonnes);

		for(index_lignes=0; index_lignes<Z_nblignes;index_lignes++)
		{
			Z.put(index_lignes,0,perf.get(index_lignes,0));
			Z.put(index_lignes,1,entree.get(index_lignes,0));
		}

		arx_bn(Z,na,nb);

		X.set_size(min_len,1);
		X.fill(0);
		X.put(0,0,1);
		bi=filter(b,a,X);
		verS=filter(b,a,entree);

		sauve=(verS-perf).fro_norm();
		eccS=sauve*sauve/min_len;

		if(eccS<ecctis)
		{
			ecctis=eccS;
			atis=a;
			btis=b;
			bitis=bi;	
			rettis=retard;
			vertis=verS;
			perftis=perf;
		}
	}

	/******************************************************************************************/
	/*******************suppression d'un retard eventuel sur la courbe d'entree****************/
	/*****************************************************************************************/

	perf.set_size(perf1_nblignes, perf1_nbcolonnes);
	perf=perf1;
	perf_size=perf.size();
	retard=0;
	while(retard<_retard_AIF)
	{
		//on retarde l'entree
		entree=entree1.extract(entree1_size-retard,1, retard,0)-entree1.get(retard,0);
	 	entree_size=(int)entree.size();

		min_len=std::min(entree_size, perf_size);

		perf=perf1.extract(min_len,1,0,0);
		entree=entree1.extract(min_len,1,0,0);

		//construction de la matrice Z
		Z_nblignes=min_len;
		Z_nbcolonnes=2;
		Z.set_size(Z_nblignes,Z_nbcolonnes);

		for(index_lignes=0; index_lignes<Z_nblignes;index_lignes++)
		{
			Z.put(index_lignes,0,perf.get(index_lignes,0));
			Z.put(index_lignes,1,entree.get(index_lignes,0));
		}

		arx_bn(Z,na,nb);

		X.set_size(min_len,1);
		X.fill(0);
		X.put(0,0,1);
		bi=filter(b,a,X);
		verS=filter(b,a,entree);
		
		sauve=(verS-perf).frobenius_norm ();
		eccS=sauve*sauve/min_len;

		if(eccS<ecctis)
		{
			ecctis=eccS;
			atis=a;
			btis=b;
			bitis=bi;	
			rettis=-retard;
			vertis=verS;
			perftis=perf;
		}
		retard++;
	}
}

/********************************vnl_matrix<double> b, vnl_matrix<double> a********************************************************************/
void IRMCarto::arx_bn(vnl_matrix<double> Z, int na, int nb)
{
	int index_lignes, index_colonnes;
	int Z_nblignes   = (int)Z.rows();
	int Z_nbcolonnes = (int)Z.cols();
	int min_len, nmax;
	int nc=na+nb;
	int A_nblignes;

	vnl_matrix<double> erreur;
	vnl_matrix<double> A;
	vnl_matrix<double> B;

	//vnl_diag_matrix<vnl_svd<double>::singval_t > W;

	/*****************************************************************************************/
	/******verification que entree et perf sont des vecteurs colonnes*************************/
	/*****************************************************************************************/
	if((Z_nbcolonnes!=2)) 
	{
		std::cout << "Erreur (3) dans le vecteur entrees dans Arx : Z_nbcolonnes " 
			   << Z_nbcolonnes << std::endl;
		exit(1);
	}

	min_len=std::min(Z_nblignes,Z_nbcolonnes);
	
	nmax=std::max(na+1,nb)-1;
	
	//construction de la matrice A
	A_nblignes=Z_nblignes-nmax;
	A.set_size(A_nblignes,nc);
	A.fill(0);
  
	for(index_lignes=0; index_lignes<A_nblignes;index_lignes++)
	{
		for(index_colonnes=0; index_colonnes<nb;index_colonnes++)
		{
			A.put(index_lignes,index_colonnes,Z.get(index_lignes+nmax-index_colonnes,1));
		}

		for(index_colonnes=0; index_colonnes<na;index_colonnes++)
		{
			A.put(index_lignes,index_colonnes+nb,Z.get(index_lignes+nmax-index_colonnes-1,0));
		}
	}

	//construction de la matrice B
	B=Z.extract(Z_nblignes-nmax,1, nmax,0);

	vnl_svd<double> svd(A, 0);
	vnl_matrix<double> X=svd.solve(B);

	//definition de a et b
	b=X.extract(1,nb,0,0);

	//a.set_size(na,1);
	a.set_size(1,na+1);
	a.put(0,0,1);

	for(index_colonnes=1;index_colonnes<na+1;index_colonnes++)  
	{
		//printf("X=%f\n",-X.get(index_lignes-1+nb,0));
		a.put(0,index_colonnes,-X.get(index_colonnes-1+nb,0));
	}
}

void IRMCarto::cbv_arma()
{
	int index_lignes, index_colonnes;
	int a_nblignes=atis.rows();
	int a_nbcolonnes=atis.cols();
	int b_nblignes=btis.rows();
	int b_nbcolonnes=btis.cols();
	double somme_a=0;
	double somme_b=0;

	for(index_lignes=0; index_lignes<a_nblignes; index_lignes++)
	{
		for(index_colonnes=0; index_colonnes<a_nbcolonnes; index_colonnes++)
		{
			somme_a=somme_a+atis.get(index_lignes,index_colonnes);
		}
	}

	for(index_lignes=0; index_lignes<b_nblignes; index_lignes++)
	{		

		for(index_colonnes=0; index_colonnes<b_nbcolonnes; index_colonnes++)
		{
			somme_b=somme_b+btis.get(index_lignes,index_colonnes);
		}
	}

	cbv=somme_b/somme_a;
	cbf=btis[0][0];
	mtt=0;

	if(abs(cbf)>0)
	{
		mtt=cbv/cbf;
	}
}
