#ifndef __IRMCarto__
#define __IRMCarto__

//------------------------------------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------------------------------------
#include "vtkImageData.h"
//#include "vtkMetaImageReader.h"
//#include "vtkMetaImageWriter.h"

#include "creaCarto.h"

#include <vector>
#include <string>
#include <algorithm>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
//#include <time.h>
//#include <ctime>
//#include <sys/time.h> 

#include <vnl/vnl_c_vector.h>

#include <vnl/vnl_random.h>
#include <vnl/vnl_inverse.h>
#include <vnl/vnl_matrix.h>
#include <vnl/vnl_matlab_print.h>
#include <vnl/algo/vnl_svd.h>
#include <vnl/algo/vnl_svd_economy.h>
#include <iostream>	//cout
#include <stdio.h>
#include <math.h>

class CREACARTO_EXPORT IRMCarto {

public:

      /*************************************************************************************************************/
      /*Constructors & Destructors**********************************************************************************/
      /*************************************************************************************************************/

	IRMCarto(double res_temp,int nbbaseline, int delay, double eps,int min, int max,double TE, double thresnoise,int retard_AIF, int retard_perf);
	
	~IRMCarto();

      /*************************************************************************************************************/
      /*Ajout, suppression, selection d'images**********************************************************************/
      /*************************************************************************************************************/

	std::vector<vtkImageData*> getVectImages();

	void addImage(vtkImageData * img);
	
	void removeImage(int selected);
	
	vtkImageData* getSelectedImage(int pos);

      /*************************************************************************************************************/
      /*Ajout, suppression, selection d'images moyennees************************************************************/
      /*************************************************************************************************************/

	std::vector<vtkImageData*> getVectImagesMoy();

	void addImageMoy(vtkImageData * img);
	
	void removeImageMoy(int selected);
	
	vtkImageData* getSelectedImageMoy(int pos);
	
      /*************************************************************************************************************/
      /*Ajout, suppression, selection des sequences de concentration************************************************/
      /*************************************************************************************************************/
	
	std::vector<vtkImageData*> getVectSeqCon();

	void addSeqCon(vtkImageData * img);
	
	void removeSeqCon(int selected);
	
	vtkImageData* getSelectedSeqCon(int pos);

      /*************************************************************************************************************/
      /*Creation d'un pointeur sur une vtkImageData*****************************************************************/
      /*************************************************************************************************************/
         	
	double * pointageImageDouble(vtkImageData *img, int index_temps, int nb_lignes, int nb_colonnes);
	
      /*************************************************************************************************************/
      /*Moyennage d'image*******************************************************************************************/
      /*************************************************************************************************************/
	
	void moyennage3x3(vtkImageData *img,vnl_matrix<double> filtre);
	
      /*************************************************************************************************************/
      /*Operations sur les matrices*********************************************************************************/
      /*************************************************************************************************************/
		
	vnl_matrix<double> vectligne2col(vnl_matrix<double> vectlignes );
		
	void maxvecteur(vnl_matrix<double> vect,double *maxValue,int *posValue);
	
	vnl_matrix<double> filter(vnl_matrix<double> b, vnl_matrix<double> a, vnl_matrix<double> x);
	
      /*************************************************************************************************************/
      /*Accesseurs et mutateurs de la classe************************************************************************/
      /*************************************************************************************************************/

	//dimension de image entree

	void setDim(vtkImageData *img);
	
	int getDim(int n);

	//dimension de image entree moyenne

	void setDimMoy(vtkImageData *img);
	
	int getDimMoy(int n);

	//dimension de image sequence de concentration

	void setDimSeqCon(vtkImageData *img);
	
	int getDimSeqCon(int n);

	//Vecteur concentration

	void setVectConcentration(vtkImageData *img, int index_colonnes, int index_lignes);
	
	vnl_matrix<double> getVectConcentration();
	
	double getconcentration(int pos);

	//mtt, cbv, cbf, max, res_tep, max, min
	
	double getmtt();

	double getcbv();

	double getcbf();

	int gettmax();

	double getrestemp();

	int getmax();
	
	int getmin();

	//matrice vertis
	
	vnl_matrix<double> getvertis();

      /*************************************************************************************************************/
      /*Calcul de la séquence de concentration a partir d'une vtkImageData*****************************************/
      /*************************************************************************************************************/
	
//	double construct_baseline(vtkImageData *img);

	vnl_matrix<double> construct_baseline(vtkImageData *img);

	void construct_seqcon(vtkImageData *img);
      
	/*************************************************************************************************************/
      /*Calcul de la SVD et des elements de la classe***************************************************************/
      /*************************************************************************************************************/
	
	vnl_matrix<double> lectureAIFdansexcel(const char* nom_fic);
	
	vnl_matrix<double> calculSVD(vnl_matrix<double> perf,vnl_matrix<double> entree);
	
	void mttr_tmax(vnl_matrix<double> r);
  
	
      /*************************************************************************************************************/
      /*Calcul de ARMA et des elements de la classe*****************************************************************/
      /*************************************************************************************************************/
		
	void calculARMA(vnl_matrix<double> perf,vnl_matrix<double> entree, int Na, int Nb);

	void arx_bn(vnl_matrix<double> Z, int na, int nb);

	void cbv_arma();

private: 
  
      /*************************************************************************************************************/
      /*Attributs***************************************************************************************************/
      /*************************************************************************************************************/
	std::vector<vtkImageData*> _vectimages;		//vecteur de vtkImagaData des images d'entree
	std::vector<vtkImageData*> _vectimagesMoy;	//vecteur de vtkImagaData des images moyennees
	std::vector<vtkImageData*> _vectseqcon;		//vecteur de vtkImagaData des sequences de concentration	
	
	//vtkImageData* _sous_image;
	
	vnl_matrix<double> _concentration;	//vecteur colonnes 
	
	int* _dimEntree;	//dimensions de l'image d'entree
	int* _dimMoy;		//dimensions de l'image d'entree moyennee
	int* _dimSeqCon;	//dimensions de l'image sequence de concentration

	int _min,_max, _delay;	//min et max: 1er et dernière images prises en compte, delay: choix du retard
	int _nbbaseline;		//nombre d'image pour la baseline
	double _res_temp;	//res_temp: serolution temporelle
	double _eps;		//
	double _thresnoise;	//
	double _TE;		//	
	int _retard_AIF;
	int _retard_perf;
	
	vnl_matrix<double>vertis;	//matrice resultant de la SVD 
	double mtt,cbv,cbf;		//paramètres resultant de la SVD	
	int tmax;			//paramètres resultant de la SVD

	//matrices utilisée pour le modèle ARMA
	vnl_matrix<double> a;		
	vnl_matrix<double> b;
	vnl_matrix<double> atis;		
	vnl_matrix<double> btis;
	
};
#endif
