#ifndef __IRMCreate__
#define __IRMCreate__

//------------------------------------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------------------------------------
#include "IRMCarto.h"
#include <vector>
#include <iostream>
#include "vtkGdcmReader.h"
#include <vtkImageData.h>
#include <vtkImageViewer.h>
#include "vtkRenderWindowInteractor.h"
#include <vtkImageViewer2.h>
#include <string>
#include <stdlib.h>
#include <stdio.h>
#include <sstream>
#include <vtkMetaImageWriter.h>
//#include <sys/time.h>
#include <math.h>


class CREACARTO_EXPORT IRMCreate {

public:

      /*************************************************************************************************************/
      /*Constructors & Destructors**********************************************************************************/
      /*************************************************************************************************************/
	IRMCreate(double res_temp, int nbbaseline, int delay, double eps, int min, int max, double TE, double thresnoise, int retard_AIF, int retard_perf);

	~IRMCreate();
	
      /*************************************************************************************************************/
      /*Ajout, suppression, selection d'images**********************************************************************/
      /*************************************************************************************************************/
	
	std::vector<vtkImageData*> getVectImages();
	
	void addImage(vtkImageData * img);
	
	void removeImage(int selected);
		
	vtkImageData * getSelectedImage(int pos);
	
	    /*************************************************************************************************************/
      /*Ajout, suppression, selection d'images moyennees************************************************************/
      /*************************************************************************************************************/

	std::vector<vtkImageData*> getVectImagesMoy();

	void addImageMoy(vtkImageData * img);
	
	void removeImageMoy(int selected);
	
	vtkImageData* getSelectedImageMoy(int pos);
	
      /*************************************************************************************************************/
      /*Ajout, suppression, selection des sequences de concentration************************************************/
      /*************************************************************************************************************/
	
	std::vector<vtkImageData*> getVectSeqCon();

	void addSeqCon(vtkImageData * img);
	
	void removeSeqCon(int selected);
	
	vtkImageData* getSelectedSeqCon(int pos);
	
    
      /*************************************************************************************************************/
      /*Accesseurs et mutateurs de la classe************************************************************************/
      /*************************************************************************************************************/	
      
	std::vector<vtkImageData*> getVectMAX();
	
	std::vector<vtkImageData*> getVectTTP();
	
	std::vector<vtkImageData*> getVectMTTSVD();
	
	std::vector<vtkImageData*> getVectCBVSVD();
	
	std::vector<vtkImageData*> getVectCBFSVD();
	
	std::vector<vtkImageData*> getVectTMAXSVD();
	
	std::vector<vtkImageData*> getVectMTTARMA();
	
	std::vector<vtkImageData*> getVectCBVARMA();
	
	std::vector<vtkImageData*> getVectCBFARMA();
	
	IRMCarto* getCarto();
	
	/*************************************************************************************************************/
	/*Moyennage des images****************************************************************************************/
	/*************************************************************************************************************/
	void MoyennageImage(int inputImage);

	/*************************************************************************************************************/
	/*Calcul des sequences de concentration***********************************************************************/
	/*************************************************************************************************************/
	void CalculSeqCon(int inputImage);
	
      /*************************************************************************************************************/
      /*Calcul des cartographies par la svd*************************************************************************/
      /*************************************************************************************************************/
      
	//void createCartoMAX_TTP(int inputImage);
	
	void createCartoMAX_TTP(int inputImage, vtkImageData *image_max, vtkImageData *image_ttp );
				
	void createCartoSVD(int inputImage, vnl_matrix<double> entree_arterielle, vtkImageData *image_mtt, vtkImageData *image_cbv, vtkImageData *image_cbf, vtkImageData *image_tmax);

	void createCartoARMA(int inputImage, vnl_matrix<double> entree_arterielle, vtkImageData *image_mtt, vtkImageData *image_cbv, vtkImageData *image_cbf);
		

      /*************************************************************************************************************/
      /*Affichage et enregistrement des cartographies***************************************************************/
      /*************************************************************************************************************/
	void affichageImage(vtkImageData *img);
	
	void enregistrerImage(std::vector<vtkImageData*> vect, char *nom, int nb);
	
	void enregistrercarto(std::vector<vtkImageData*> vect, char *nom, int nb);
	
private: 	
  
      /*************************************************************************************************************/
      /*Attributs***************************************************************************************************/
      /*************************************************************************************************************/
      
	IRMCarto *carto;
	
	std::vector<vtkImageData*> VectMAX;
	std::vector<vtkImageData*> VectTTP;
	std::vector<vtkImageData*> VectMTTSVD;
	std::vector<vtkImageData*> VectCBVSVD;
	std::vector<vtkImageData*> VectCBFSVD;
	std::vector<vtkImageData*> VectTMAXSVD;
	std::vector<vtkImageData*> VectMTTARMA;
	std::vector<vtkImageData*> VectCBVARMA;
	std::vector<vtkImageData*> VectCBFARMA;
	
};
#endif
