#pragma once

#include <wx/wx.h>
#include <string>

#include "panelTools.h"
#include "buttonsIO.h"
#include "buttonsEdition.h"
#include "buttonsIRM.h"
#include "buttonsContours.h"
#include "menuBarButtons.h"

#include <wx/treebook.h>
#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/stattext.h>
#include <wx/splitter.h>
#include <wx/statline.h>

#include "vtkImageData.h"

#include "IRMMainFrame.h"

class buttonsMainPanel : public wxPanel 
{

public:
	
	buttonsMainPanel(wxWindow * parent, int sizex, int sizey, wxEvtHandler * evtHandler, std::string datadir = "data/Icons");
	~buttonsMainPanel(void);

	/**	This method should be called the first time by the parent class
	**	in creates all the panels the first time
	**/
	static buttonsMainPanel* getInstance(wxWindow* parent, std::string datadir, wxEvtHandler * evtHandler=NULL );
	/**	This method should be called the first time by the parent class
	**	in creates all the panels the first time
	**/
	static buttonsMainPanel* getInstance(wxWindow* parent, wxEvtHandler * evtHandler=NULL);
	/**
	**	Gets the instance of the class
	**/
	static buttonsMainPanel* getInstance();

	/**
	**	Responds to the events of the buttons, when the same panel is responsible for it. It gets the name
	**	of the button corresponding to the method that has to be executed,
	**	it uses attribute eventHandler to call the methods define by the application.
	**	see setEventHandler(wxEventHandler*)
	**/
	void onActionButtonPressed1( wxCommandEvent& event );

	/**
	**	Shows the panel in the 
	**/
	void showPanel(wxWindow* panel);
	/**
	**	Gets the Information panel
	**/
	wxPanel* getInfoPanel();
	
	/**
	**	This methods connect the event in the class interfNewContourMenu with the world of the appli
	**/

	void onDifferential();
	void onAverage(int selected);
	void onDifferentialPressed();
	void onSuperposition(int img1, int img2);
	void onSuperpositionPressed();
	void onSignalCurve();
	void onSignalCurvePressed();
	/////////////////////////
	void onCopy();
	void onPaste();
	void onUndo();
	void onRedo();

	//////////////////////////
	void onNewADC();
	void onNewADCPressed();
	void onNewTTP();
	void onNewTTPPressed();
	void onSave();
	void onSaveResults(std::string directory,std::string namefile, std::string filename,int typeContourGroup);
	/////////////////////////////
	void RefreshInterface();
	void onViewerPressed();
	void onMIPViewerPressed();
	
	void onContour(int selected);
	void onContourPressed();

	int numberOfViewers();
	int selectedView();
	int getDimZ(int selected);
	void updateViewersInfo();
	void changeViewer(int sel);
	void createADC(int inputImage, std::string path, int min, int max, bool addToDB);

	void createTTP ( std::string path, int min, int max, std::string ficEntreeArt,
			double res_temp, int nbbaseline, int delay, double eps, double TE, double thresnoise, int retard_AIF, int retard_perf,
			bool verbose, 
			bool cartoMax, 
			bool cartoSVD, 
			bool cartoARMA,
			bool aff,
			bool enreg,
			bool addToDB,
			int firstSliceToDeal); 						// JPR	
	
	void showWindow();
	void hideWindow();

private:

	static buttonsMainPanel* mainPanel;

	wxPanel* menubar1;
	wxPanel* menubar2;
	wxPanel* menubar3;
	wxPanel* menubar4;

	wxWindow* currentWindow;
	wxSplitterWindow* splitterwindow;

	wxPanel* infoPanel;

	wxStaticText* statictext1;
	wxStaticText* statictext2;
	wxStaticText* statictext3;
	wxStaticText* statictext4;

	std::string PANEL1;
	std::string PANEL2;
	std::string PANEL3;
	std::string PANEL4;

	wxPanel* differentialPanel;
	interfDifferentialPanel* panelDifferential;	

	wxPanel* superpositionPanel;
	interfSuperpositionPanel* panelSuperposition;	

	wxPanel* signalCurvePanel;
	interfSignalCurvePanel* panelSignalCurve;	

	wxPanel* contourPanel;
	interfContourPanel* panelContour;

	wxPanel* ADCPanel;
	interfADCPanel* panelADC;

	wxPanel* TTPPanel;
	interfTTPPanel* panelTTP;

	/**
	**	initialize a statictext component of the interface
	**/
	wxStaticText* getText(wxWindow* parent, std::string nom);
	/**
	** initializes a button with data
	**/
	wxButton* getButton(wxWindow* parent);
	/**
	** Puts the panel button and the text in the sizer
	**/
	wxPanel* setMenuTextSizer(wxWindow* parent, wxPanel* panel, wxStaticText* text);	
};
