#include "panelTools.h"
#include "buttonsMainPanel.h"
#include "creaWx.h"
#include <fstream>
#include <string>
#include <stdlib.h>  // for atoi

/**
** Begin of the differential panel
**/

interfDifferentialPanel::interfDifferentialPanel(wxWindow* parent)
: wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN)
{
	wxButton *aveGoBtn =          new wxButton(this, -1, _T("Get Average"),          wxDefaultPosition, wxSize(200,30) );
	wxButton *differentialGoBtn = new wxButton(this, -1, _T("Generate Cartography"), wxDefaultPosition, wxSize(200,30) );

	wxString viewers[2];
	viewers[0]= _T("Viewer 1");
	viewers[1]= _T("Viewer 2");

	wxString imageType[2];
	imageType[0]= _T("TTP");
	imageType[1]= _T("ADC");

	_boxViewer = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,25), 2, viewers,   wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	_boxType   = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,25), 2, imageType, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);

	_aveValue = new wxStaticText(this, -1, _T(" "));

	Connect( aveGoBtn->GetId(),          wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfDifferentialPanel::onAverageGo ); 
	Connect( differentialGoBtn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfDifferentialPanel::onDifferentialGo ); 

	wxFlexGridSizer *sizer = new wxFlexGridSizer(1);
	sizer->Add( new wxStaticText(this, -1, _T("Viewer to Apply")), 1, wxGROW );
	sizer->Add( _boxViewer, 1, wxGROW );
	sizer->Add( aveGoBtn, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T(" ")),               1, wxGROW );
	sizer->Add( _aveValue, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T(" ")),               1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T("Type of Image")),   1, wxGROW );
	sizer->Add( _boxType, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1 , _T(" ")),              1, wxGROW );
	sizer->Add( differentialGoBtn, 1, wxGROW );

	this->SetSizer( sizer );
	this->SetAutoLayout( true );
	this->Layout();
}

interfDifferentialPanel::~interfDifferentialPanel()
{
}

void interfDifferentialPanel::onAverageGo(wxCommandEvent& event)
{
	_boxViewer->GetValue();
	int selected = -1;
	if (_boxViewer->GetValue()==_T("Viewer 1"))
	{
		selected=1;
	}
	else if (_boxViewer->GetValue()==_T("Viewer 2"))
	{
		selected=2;
	}

	buttonsMainPanel::getInstance()->onAverage(selected);
	//_aveValue->SetLabel(_T("Average Value: 157"));                // ?!? JPR
	_aveValue->SetLabel(_T("Average Value: Not Yet Implemented"));  //  JPR
}

void interfDifferentialPanel::onDifferentialGo(wxCommandEvent& event)
{
	//wxString thick=_thickness->GetValue();
	buttonsMainPanel::getInstance()->onDifferential();
}

/**
** Begin of the superposition panel
**/

interfSuperpositionPanel::interfSuperpositionPanel(wxWindow* parent)
: wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN)
{
	wxButton *superposeGoBtn      = new wxButton(this, -1, _T("Superpose"),                 wxDefaultPosition, wxSize(100,30) );
	wxButton *alignGoBtn          = new wxButton(this, -1, _T("Align"),                     wxDefaultPosition, wxSize(150,30) );
	wxButton *saveParametersGoBtn = new wxButton(this, -1, _T("Save Alignment Parameters"), wxDefaultPosition, wxSize(150,30) );

	_boxImage1 = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(50,25), 0, NULL, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	_boxImage2 = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(50,25), 0, NULL, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	updateViewersList();
	
	Connect( alignGoBtn->GetId(),          wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfSuperpositionPanel::onAlignGo );
	Connect( superposeGoBtn->GetId(),      wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfSuperpositionPanel::onSuperposeGo );
	Connect( saveParametersGoBtn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfSuperpositionPanel::onSaveParametersGo );

	wxFlexGridSizer * sizer		= new wxFlexGridSizer(2,5);
	sizer->Add( new wxStaticText(this, -1, _T("Image 1:		")), 1, wxGROW );
	sizer->Add( _boxImage1, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T("Image 2:		")), 1, wxGROW );	
	sizer->Add( _boxImage2, 1, wxGROW );
	sizer->Add( superposeGoBtn, 1, wxGROW );
	sizer->Add( alignGoBtn, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T(" ")),                     1, wxGROW );
	sizer->Add( saveParametersGoBtn, 1, wxGROW );

	this->SetSizer( sizer );
	this->SetAutoLayout( true );
	this->Layout();
}

interfSuperpositionPanel::~interfSuperpositionPanel()
{

}

void interfSuperpositionPanel::onSuperposeGo(wxCommandEvent& event)
{
	int img1 = _boxImage1->GetCurrentSelection();
	int img2 = _boxImage2->GetCurrentSelection();

	buttonsMainPanel::getInstance()->onSuperposition(img1, img2);
}

void interfSuperpositionPanel::onAlignGo(wxCommandEvent& event)
{

}

void interfSuperpositionPanel::onSaveParametersGo(wxCommandEvent& event)
{

}

void interfSuperpositionPanel::updateViewersList()
{
	_boxImage1->Clear();
	_boxImage2->Clear();

	int nView = buttonsMainPanel::getInstance()->numberOfViewers();
	for (int i=1; i<=nView; i++)
	{
		std::ostringstream name;
		name << "Viewer " << i;
		_boxImage1->Insert(crea::std2wx(name.str()), i-1);
		_boxImage2->Insert(crea::std2wx(name.str()), i-1);
	}
}

/**
** Begin of the signal curve panel
**/

interfSignalCurvePanel::interfSignalCurvePanel(wxWindow* parent)
: wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN)
{
	wxButton *signalCurveGoBtn = new wxButton(this,-1,_T("Create Signal Curve"), wxDefaultPosition, wxSize(130,30) );

	wxString day[7];
	day[0]= _T("1");
	day[1]= _T("2");
	day[2]= _T("3");
	day[3]= _T("4");
	day[4]= _T("5");
	day[5]= _T("6");
	day[6]= _T("7");

	wxString slice[8];
	slice[0]= _T("1");
	slice[1]= _T("2");
	slice[2]= _T("3");
	slice[3]= _T("4");
	slice[4]= _T("5");
	slice[5]= _T("6");
	slice[6]= _T("7");
	slice[7]= _T("8");

	_boxDay   = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(130,25), 7, day,   wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	_boxSlice = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(130,25), 8, slice, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	
	Connect( signalCurveGoBtn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfSignalCurvePanel::onSignalCurveGo ); 

	//wxPoint(50,40)InterfaceButtons/panelTools.cxx

	wxFlexGridSizer *sizer = new wxFlexGridSizer(2,3);
	sizer->Add( new wxStaticText(this, -1, _T("Day:       ")), 1, wxGROW );
	sizer->Add( _boxDay, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T("Slice:     ")), 1, wxGROW );
	sizer->Add( _boxSlice, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1 ,_T(" ")),           1, wxGROW );
	sizer->Add( signalCurveGoBtn, 1, wxGROW );

	this->SetSizer( sizer );
	this->SetAutoLayout( true );
	this->Layout();
}

interfSignalCurvePanel::~interfSignalCurvePanel()
{
}

void interfSignalCurvePanel::onSignalCurveGo(wxCommandEvent& event)
{
	//wxString thick=_thickness->GetValue();
	buttonsMainPanel::getInstance()->onSignalCurve();
}

/**
** Begin of the contour panel
**/
interfContourPanel::interfContourPanel(wxWindow* parent)
: wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN)
{
	wxButton *openBtn = new wxButton(this,-1,_T("Open"), wxDefaultPosition, wxSize(200,30) );
	_boxViewer = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,25), 0, NULL, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	updateViewersList();

	Connect( openBtn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED , (wxObjectEventFunction) &interfContourPanel::onContourGo ); 

	wxFlexGridSizer * sizer	= new wxFlexGridSizer(1);
	sizer->Add( new wxStaticText(this, -1, _T("Viewer to Load")), 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T(" ")),              1, wxGROW );
	sizer->Add( _boxViewer, 1, wxGROW );
	sizer->Add( openBtn, 1, wxGROW );

	this->SetSizer( sizer );
	this->SetAutoLayout( true );
	this->Layout();
}

interfContourPanel::~interfContourPanel()
{
}

void interfContourPanel::onViewChange(wxCommandEvent& event)
{
	int sel = _boxViewer->GetCurrentSelection();
	buttonsMainPanel::getInstance()->changeViewer(sel);
}

void interfContourPanel::onContourGo(wxCommandEvent& event)
{
	int selected = _boxViewer->GetCurrentSelection ();
	//std::cout<<"Seleccionado: "<<selected<<std::endl;
	buttonsMainPanel::getInstance()->onContour(selected);
}

void interfContourPanel::updateViewersList()
{
	_boxViewer->Clear();

	int nView = buttonsMainPanel::getInstance()->numberOfViewers();
	for (int i=1; i<=nView; i++)
	{
		std::ostringstream name;
		name << "Viewer " << i;
		//std::cout<<name.str()<<std::endl;
		//_boxViewer->Insert(_T(name.str()), i-1);
		_boxViewer->Insert(crea::std2wx(name.str()), i-1);
	}
	int selected = buttonsMainPanel::getInstance()->selectedView();
	_boxViewer->Select(selected);
}

/**
** Begin of the ADC panel
**/
interfADCPanel::interfADCPanel(wxWindow* parent)
    : wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN)
{
	_boxViewer = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,25), 0, NULL, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	updateViewersList();

	//wxButton *getSelected = new wxButton(this, wxID_ANY, _T("Get"), wxDefaultPosition, wxSize(50,30) );  // added and commented out JPR

        _outputPath = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,30));
	wxButton *outputDir = new wxButton(this, wxID_ANY, _T("Output Directory"), wxDefaultPosition, wxSize(120,30) );

	int min = 0;
	int max = 100;
	if(buttonsMainPanel::getInstance()->numberOfViewers()>0)
	{
		max = buttonsMainPanel::getInstance()->getDimZ(_boxViewer->GetCurrentSelection());
	}

	_mBarSlices = new mBarRange(this,70,65);
	_mBarSlices->SetMin(0);
	_mBarSlices->SetStart(0);
	_mBarSlices->SetOrientation(true);
	_mBarSlices->setActiveStateTo(true);
	_mBarSlices->setVisibleLabels(true);
	_mBarSlices->setDeviceEndMargin(10);
	_mBarSlices->setDeviceBlitStart(10, 10);
	_mBarSlices->setIfWithActualDrawed(false);
   	_mBarSlices->setRepresentedValues(min, max);	
	_mBarSlices->SetStart(min);
	_mBarSlices->SetEnd(max); 
	updateSlices();

	_addCheckBox = new wxCheckBox(this, -1, _T("Add Images to Database?") );
	_addCheckBox->SetValue(true);

	wxButton *createBtn = new wxButton(this,-1, _T("Generate Cartography"), wxDefaultPosition, wxDefaultSize );

	Connect( outputDir->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfADCPanel::onOutputDir );
	Connect( createBtn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfADCPanel::onCreateADC );
	//Connect( getSelected->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfADCPanel::onGetSelected ); // added and commented outJP

	wxFlexGridSizer *sizer	= new wxFlexGridSizer(1, 7);
	sizer->Add( new wxStaticText(this, -1, _T("Input Image")), 1, wxGROW );
	sizer->Add( _boxViewer, 1, wxGROW );
	sizer->Add( new wxStaticText(this, -1, _T("Output directory")), 1, wxGROW );

	wxFlexGridSizer *sizerB = new wxFlexGridSizer(2, 1);
	sizerB->Add( _outputPath, 1, wxGROW );
	sizerB->Add( outputDir,   1, wxGROW );
	sizer->Add( sizerB,       1, wxGROW );

	sizer->Add( new wxStaticText(this,-1,_T("Selected Slices")), 1, wxGROW );
	sizer->Add( _mBarSlices,  1, wxGROW );
	sizer->Add( _addCheckBox, 1, wxGROW );
	//sizer->Add( _armaCheckBox, 1, wxGROW );	
	sizer->Add( createBtn,    1, wxGROW );

	this->SetSizer(sizer);
	this->SetAutoLayout(true);
	this->Layout();
}

interfADCPanel::~interfADCPanel()
{

}

void interfADCPanel::onGetSelected(wxCommandEvent& event) // JPR
{
std::cout << "-------------> interfADCPanel::onGetSelected " << std::endl;

}

void interfADCPanel::onOutputDir(wxCommandEvent& event)
{
	long style = wxDD_DEFAULT_STYLE;
	wxDirDialog* FD = new wxDirDialog( 0, _T("Select the Output Directory"), wxEmptyString, style);
	   
	if (FD->ShowModal()==wxID_OK)
	{
		_outputPath->SetValue(FD->GetPath());
	}
}

void interfADCPanel::onCreateADC(wxCommandEvent& event)
{
	int inputImage   = _boxViewer->GetCurrentSelection();
	std::string path = crea::wx2std(_outputPath->GetValue());
	int min          = _mBarSlices->GetStart();
	int max          = _mBarSlices->GetEnd();
	bool addToDB     = _addCheckBox->GetValue();

	std::cout << "=======================================addToDB " << addToDB << std::endl;

 /// \TODO call Solene's function
    buttonsMainPanel::getInstance()->createADC(inputImage, path, min, max, addToDB);
	//buttonsMainPanel::getInstance()->createADC(inputPath, outputPath, min, max, addToDB);
}

void interfADCPanel::onSlidesChange(wxCommandEvent& event)
{
	//Catch the Event of the mBarRange
}

void interfADCPanel::onViewChange(wxCommandEvent& event)
{
	int sel = _boxViewer->GetCurrentSelection();
	buttonsMainPanel::getInstance()->changeViewer(sel);
}

void interfADCPanel::updateViewersList()
{
	_boxViewer->Clear();

	int nView = buttonsMainPanel::getInstance()->numberOfViewers();
	for (int i=1; i<=nView; i++)
	{
		std::ostringstream name;
		name << "Viewer " << i;
		//std::cout<<name.str()<<std::endl;
		
		std::string sstmpStr = name.str();
		_boxViewer->Insert(crea::std2wx(sstmpStr.c_str()), i-1);
	}
	int selected = buttonsMainPanel::getInstance()->selectedView();
	_boxViewer->Select(selected);
}

void interfADCPanel::updateSlices()
{	
	int min = 0;
	int max = 100;
	if(buttonsMainPanel::getInstance()->numberOfViewers()>0)
	{
        max = (buttonsMainPanel::getInstance()->getDimZ(_boxViewer->GetCurrentSelection()))-1;
	}

   	_mBarSlices->setRepresentedValues( min, max );	
	_mBarSlices->SetStart( min );
	_mBarSlices->SetEnd( max );  
}

// ==================================================================================================================
/**
** Begin of the TTP panel
**/
interfTTPPanel::interfTTPPanel(wxWindow* parent)
    : wxPanel(parent, -1, wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN)
{
	//_boxViewer = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(150,25), 0, NULL, wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB);
	//updateViewersList();


/*
	float res_temp
	int nbbaseline = 3;   // =   am->ArgMgrGetInt("nb_baseline",3);
	int delay = 0; 	      // =   am->ArgMgrGetInt("delay",0);
	float eps = 0.01;     // =   am->ArgMgrGetFloat("eps",0.1);
	//int min; 	      // =   am->ArgMgrGetInt("min",8);
	//int max;            // =   am->ArgMgrGetInt("max",100);
	float TE = 0.05;      // =   am->ArgMgrGetFloat("TE",0.05);
	int thresnoise = 600; // =   am->ArgMgrGetInt("thresnoise",600);
	int retard_AIF = 1;   // =   am->ArgMgrGetInt("retard_AIF",1);
	int retard_perf = 3;  // =   am->ArgMgrGetInt("retard_perf",3);
*/



	int min = 0;
	int max = 100;

//EED 03-11-2009
//	if(buttonsMainPanel::getInstance()->numberOfViewers()>0)
//	{
//		max = buttonsMainPanel::getInstance()->getDimZ(_boxViewer->GetCurrentSelection());
//	}

	wxFlexGridSizer *sizer	= new wxFlexGridSizer(1);  // 7 --> 8 // JPR

	sizer->Add( new wxStaticText(this, -1, _T("Output directory")), 1, wxGROW );

	wxFlexGridSizer *sizerB = new wxFlexGridSizer(2, 1); 
			
	_outputPath = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(200,30));
	wxButton *outputDir = new wxButton(this, wxID_ANY, _T("Choose"), wxDefaultPosition, wxSize(70,30) );	
	sizerB->Add( _outputPath, 1, wxGROW );
	sizerB->Add( outputDir,   1, wxGROW );
	
	sizer->Add( sizerB,       1, wxGROW );
	
	_mBarSlices = new mBarRange(this,70,65);
	_mBarSlices->SetMin(0);
	_mBarSlices->SetStart(0);
	_mBarSlices->SetOrientation( true );
	_mBarSlices->setActiveStateTo(true);
	_mBarSlices->setVisibleLabels( true );
	_mBarSlices->setDeviceEndMargin(10);
	_mBarSlices->setDeviceBlitStart(10, 10);
	_mBarSlices->setIfWithActualDrawed( false );
   	_mBarSlices->setRepresentedValues( min , max );	
	_mBarSlices->SetStart( min );
	_mBarSlices->SetEnd( max ); 

//EED 03-11-2009
//	updateSlices();

	sizer->Add( new wxStaticText(this,-1,_T("Selected Slices")),    1, wxGROW );
	sizer->Add( _mBarSlices,   1, wxGROW );
		
	_addCheckBox = new wxCheckBox(this, -1, _T("Add Images to Database?") );
	_addCheckBox->SetValue(true);	
	sizer->Add( _addCheckBox,  1, wxGROW );
	
	_armaCheckBox = new wxCheckBox(this, -1, _T("Compute Arma Carto?") );
	_armaCheckBox->SetValue(false);	
	sizer->Add( _armaCheckBox,  1, wxGROW );	

	// ----							pour res_temp
	wxFlexGridSizer *sizerC = new wxFlexGridSizer(2, 1); 
	_res_temp_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("1.0"), wxDefaultPosition, wxSize(80,30));
	sizerC->Add( _res_temp_TextCtrl, 1, wxGROW );
	sizerC->Add( new wxStaticText(this,-1,_T("res_temp")),    1, wxGROW );
	sizer->Add( sizerC,        1, wxGROW ); 
	
	// ----								pour nbbaseline
	wxFlexGridSizer *sizerD = new wxFlexGridSizer(2, 1); 
	_nbbaseline_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("3"), wxDefaultPosition, wxSize(80,30));
	sizerD->Add( _nbbaseline_TextCtrl, 1, wxGROW );
	sizerD->Add( new wxStaticText(this,-1,_T("nbbaseline")),    1, wxGROW );
	sizer->Add( sizerD,        1, wxGROW ); 

	// ----								pour delay
	wxFlexGridSizer *sizerE = new wxFlexGridSizer(2, 1); 
	_delay_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("0"), wxDefaultPosition, wxSize(80,30));
	sizerE->Add( _delay_TextCtrl, 1, wxGROW );
	sizerE->Add( new wxStaticText(this,-1,_T("delay")),    1, wxGROW );
	sizer->Add( sizerE,        1, wxGROW ); 

	// ----								pour eps
	wxFlexGridSizer *sizerF = new wxFlexGridSizer(2, 1); 
	_eps_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("0.1"), wxDefaultPosition, wxSize(80,30));
	sizerF->Add( _eps_TextCtrl, 1, wxGROW );
	sizerF->Add( new wxStaticText(this,-1,_T("eps")),    1, wxGROW );
	sizer->Add( sizerF,        1, wxGROW ); 

	// ----								pour TE
	wxFlexGridSizer *sizerG = new wxFlexGridSizer(2, 1); 
	_TE_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("0.05"), wxDefaultPosition, wxSize(80,30));
	sizerG->Add( _TE_TextCtrl, 1, wxGROW );
	sizerG->Add( new wxStaticText(this,-1,_T("TE")),    1, wxGROW );
	sizer->Add( sizerG,        1, wxGROW ); 

	// ----								pour thresnoise
	wxFlexGridSizer *sizerH = new wxFlexGridSizer(2, 1); 
	_thresnoise_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("600"), wxDefaultPosition, wxSize(80,30));
	sizerG->Add( _thresnoise_TextCtrl, 1, wxGROW );
	sizerG->Add( new wxStaticText(this,-1,_T("thresnoise")),    1, wxGROW );
	sizer->Add( sizerH,        1, wxGROW );

	// ----								pour retard_AIF
	wxFlexGridSizer *sizerI = new wxFlexGridSizer(2, 1); 
	_retard_AIF_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("1"), wxDefaultPosition, wxSize(80,30));
	sizerI->Add( _retard_AIF_TextCtrl, 1, wxGROW );
	sizerI->Add( new wxStaticText(this,-1,_T("retard_AIF")),    1, wxGROW );
	sizer->Add( sizerI,        1, wxGROW );	

	// ----								pour retard_perf
	wxFlexGridSizer *sizerJ = new wxFlexGridSizer(2, 1); 
	_retard_perf_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("3"), wxDefaultPosition, wxSize(80,30));
	sizerJ->Add( _retard_perf_TextCtrl, 1, wxGROW );
	sizerJ->Add( new wxStaticText(this,-1,_T("retard_perf")),    1, wxGROW );
	sizer->Add( sizerJ,        1, wxGROW );

	// ----								pour firstSliceToDeal
	wxFlexGridSizer *sizerK = new wxFlexGridSizer(2, 1);
	_firstSliceToDeal_TextCtrl = new wxTextCtrl(this, wxID_ANY, _T("0"), wxDefaultPosition, wxSize(80,30));
	sizerK->Add( _firstSliceToDeal_TextCtrl, 1, wxGROW );
	sizerK->Add( new wxStaticText(this,-1,_T("firstSliceToDeal")),    1, wxGROW );
	sizer->Add( sizerK,        1, wxGROW );

	wxButton *createBtn = new wxButton(this,-1, _T("Generate Cartography"), wxDefaultPosition, wxDefaultSize );
	sizer->Add( createBtn,     1, wxGROW );	

	Connect( outputDir->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfTTPPanel::onOutputDir );
	Connect( createBtn->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction) &interfTTPPanel::onCreateTTP );

	this->SetSizer( sizer );
	this->SetAutoLayout( true );
	this->Layout();
}

interfTTPPanel::~interfTTPPanel()
{

}

void interfTTPPanel::onGetSelected(wxCommandEvent& event) // JPR
{
   std::cout << "-------------> interfTTPPanel::onGetSelected " << std::endl;
}

void interfTTPPanel::onOutputDir(wxCommandEvent& event)
{
	long style = wxDD_DEFAULT_STYLE;
	wxDirDialog* FD = new wxDirDialog( 0, _T("Select the Output Directory"), wxEmptyString, style);
	   
	if (FD->ShowModal()==wxID_OK)
	{
		_outputPath->SetValue(FD->GetPath());
	}
}

void interfTTPPanel::onCreateTTP(wxCommandEvent& event)
{

std::cout << "in interfTTPPanel::onCreateTTP, initialise parameters from interface\n" << std::endl;;

	//int inputImage   = _boxViewer->GetCurrentSelection();

	std::vector<std::string>inputImageS; /// \TODO : Get it from the interface ! 
	std::string ficEntreeArt("fichier_entree_arterielle.csv");  /// \TODO : Get it from the interface !

	std::string path = crea::wx2std(_outputPath->GetValue());
	int min          = _mBarSlices->GetStart();
	int max          = _mBarSlices->GetEnd();
	bool addToDB     = _addCheckBox->GetValue();
	bool cartoARMA   = _armaCheckBox->GetValue();
	float res_temp   = atof ( crea::wx2std(_res_temp_TextCtrl->GetValue()).c_str() );	
		std::cout << "=============================== res_temp " << res_temp << std::endl;
	int nbbaseline   = atoi ( crea::wx2std(_nbbaseline_TextCtrl->GetValue()).c_str() );
		std::cout << "=============================== nbbaseline " << nbbaseline << std::endl;
	int delay        = atoi ( crea::wx2std(_delay_TextCtrl->GetValue()).c_str() );
		std::cout << "=============================== delay " << delay << std::endl;
	float eps        = atof ( crea::wx2std(_eps_TextCtrl->GetValue()).c_str() );
		std::cout << "=============================== eps " << eps << std::endl;
	float TE         = atof ( crea::wx2std(_TE_TextCtrl->GetValue()).c_str() );
		std::cout << "=============================== TE " << TE << std::endl;
	int thresnoise   = atoi ( crea::wx2std(_thresnoise_TextCtrl->GetValue()).c_str() );
		std::cout << "=============================== thresnoise " << thresnoise << std::endl;			
	int retard_AIF   = atoi ( crea::wx2std(_retard_AIF_TextCtrl->GetValue()).c_str() );
		std::cout << "=============================== retard_AIF  " << retard_AIF  << std::endl;
	int retard_perf  = atoi ( crea::wx2std(_retard_perf_TextCtrl->GetValue()).c_str() );	
		std::cout << "=============================== retard_perf  " << retard_perf  << std::endl;
	int firstSliceToDeal  = atoi ( crea::wx2std(_firstSliceToDeal_TextCtrl->GetValue()).c_str() );	
		std::cout << "=============================== firstSliceToDeal  " << firstSliceToDeal  << std::endl;						
 /// call Solene's function
    //buttonsMainPanel::getInstance()->createTTP(inputImage, path, min, max, addToDB);
    
    ///  get the initial values from the interface! // JPR

	//float res_temp = 1.;  // =   am->ArgMgrGetFloat("res_temp",1.);
	//int nbbaseline = 3;   // =   am->ArgMgrGetInt("nb_baseline",3);
	//int delay = 0; 	      // =   am->ArgMgrGetInt("delay",0);
	//float eps = 0.01;     // =   am->ArgMgrGetFloat("eps",0.1);
	//int min; 	      // =   am->ArgMgrGetInt("min",8);
	//int max;            // =   am->ArgMgrGetInt("max",100);
	//float TE = 0.05;      // =   am->ArgMgrGetFloat("TE",0.05);
	//int thresnoise = 600; // =   am->ArgMgrGetInt("thresnoise",600);
	//int retard_AIF = 1;   // =   am->ArgMgrGetInt("retard_AIF",1);
	//int retard_perf = 3;  // =   am->ArgMgrGetInt("retard_perf",3);
	//int numcoupe = 0;   // =   am->ArgMgrGetInt("num_coupe",0);   // = firstSliceToDeal // JPR

        //char *ficEntreeArt;    // JPR
   	bool verbose = false;    // pas un param
        //const char *cartoMax; 
	//cartoMax = am->ArgMgrGetString("cartoPeak","O");
	bool cartoMax = true;
	//const char *cartoSVD;
	//cartoSVD=am->ArgMgrGetString("cartoSVD","O");
	bool cartoSVD = false;
	//const char *cartoARMA;
	//cartoARMA=am->ArgMgrGetString("cartoARMA","N");
	//bool cartoARMA = false;
	//const char *aff;
	//aff=am->ArgMgrGetString("affichage","O");

	//bool aff = true;
								bool aff = false; // infinite loop if true
	//const char *enreg;
	//enreg=am->ArgMgrGetString("enregistrement","O");
	bool enreg = true;

        buttonsMainPanel::getInstance()->createTTP(
			 path, 
			 min, 
			 max, 
			 ficEntreeArt,
			 res_temp, 
			 nbbaseline, 
			 delay,
			 eps, 
			 TE, 
			 thresnoise, 
			 retard_AIF, 
			 retard_perf,
			 verbose, 
			 cartoMax, 
			 cartoSVD, 
			 cartoARMA,
			 aff,
			 enreg,
			 addToDB,
			 firstSliceToDeal
			);   
}

void interfTTPPanel::onSlidesChange(wxCommandEvent& event)
{
	//Catch the Event of the mBarRange
}

void interfTTPPanel::onViewChange(wxCommandEvent& event)
{
	int sel = _boxViewer->GetCurrentSelection();
	buttonsMainPanel::getInstance()->changeViewer(sel);
}

/* EED 03-11-2009
void interfTTPPanel::updateViewersList()
{
	_boxViewer->Clear();
	int nView = buttonsMainPanel::getInstance()->numberOfViewers();
	for (int i=1; i<=nView; i++)
	{
		std::ostringstream name;
		name << "Viewer " << i;
		//std::cout<<name.str()<<std::endl;
		std::string sstmpStr = name.str();
		_boxViewer->Insert(crea::std2wx(sstmpStr.c_str()), i-1);
	}
	int selected = buttonsMainPanel::getInstance()->selectedView();
	_boxViewer->Select(selected);
}

void interfTTPPanel::updateSlices()
{	
	int min = 0;
	int max = 100;
	if(buttonsMainPanel::getInstance()->numberOfViewers()>0)
	{
        max = (buttonsMainPanel::getInstance()->getDimZ(_boxViewer->GetCurrentSelection()))-1;
	}
   	_mBarSlices->setRepresentedValues( min, max );	
	_mBarSlices->SetStart( min );
	_mBarSlices->SetEnd( max );  
}
*/

// ==================================================================================================================

BEGIN_EVENT_TABLE(interfContourPanel, wxPanel)
	EVT_COMBOBOX(wxID_ANY, interfContourPanel::onViewChange)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(interfADCPanel, wxPanel)
	EVT_COMMAND (wxID_ANY, wxEVT_TSBAR_START, interfADCPanel::onSlidesChange)
	EVT_COMMAND (wxID_ANY, wxEVT_TSBAR_END,   interfADCPanel::onSlidesChange)
	EVT_COMMAND (wxID_ANY, wxEVT_TSBAR_MOVED, interfADCPanel::onSlidesChange)	
	EVT_COMBOBOX(wxID_ANY,                    interfADCPanel::onViewChange)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(interfTTPPanel, wxPanel)
	EVT_COMMAND( wxID_ANY, wxEVT_TSBAR_START, interfTTPPanel::onSlidesChange)
	EVT_COMMAND (wxID_ANY, wxEVT_TSBAR_END,   interfTTPPanel::onSlidesChange)
	EVT_COMMAND (wxID_ANY, wxEVT_TSBAR_MOVED, interfTTPPanel::onSlidesChange)	
	EVT_COMBOBOX(wxID_ANY,                    interfTTPPanel::onViewChange)
END_EVENT_TABLE()
