//----------------------------------------------------------------------------------------------------------------
// Class definition include
//----------------------------------------------------------------------------------------------------------------
#include "IRMMainFrame.h"
#include "creaWx.h"

//----------------------------------------------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------------------------------------------

#include "wx/artprov.h"
//#include <creaWx.h>

//----------------------------------------------------------------------------------------------------------------
// Class implementation
//----------------------------------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------
	// Constructors & Destructors
	//------------------------------------------------------------------------------------------------------------

IRMMainFrame* IRMMainFrame::instance = NULL;

IRMMainFrame::IRMMainFrame(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style, std::string datadir)		
		: wxPanel(parent, id, pos, size, style)
	{
		m_mgr.SetManagedWindow(this);

		_appCore = new IRMCore (datadir);
		_gimmickPanel = NULL;
		_creaContours = NULL;
		_superpositionPanel = NULL;
		_contourID = -1;
		_superpositionID = -1;
		_datadir = datadir;

		// set up default notebook style
		m_notebook_style =wxAUI_NB_TAB_SPLIT | wxAUI_NB_TAB_EXTERNAL_MOVE | wxNO_BORDER | wxAUI_NB_CLOSE_ON_ACTIVE_TAB;
		m_notebook_theme = 0;
		_notebook = this->createNotebook();

		this->createGimmick( );

		_pannew = buttonsMainPanel::getInstance(this,_datadir+"/data/Icons");//, eventHandler);
	}

	IRMMainFrame::~IRMMainFrame()
	{
		  m_mgr.UnInit();
	}

	IRMMainFrame* IRMMainFrame::getInstance(wxWindow* parent, wxWindowID id, const wxString& title, 
					const wxPoint& pos, const wxSize& size, long style, std::string datadir)
	{
		if(instance == NULL)
		{
			instance = new IRMMainFrame(parent, id, title, pos, size, style, datadir);
		}
		return instance;
	}

	IRMMainFrame* IRMMainFrame::getInstance()
	{
		return instance;	
	}

	//------------------------------------------------------------------------------------------------------------
	// Creation and initialization methods using WxAui
	//------------------------------------------------------------------------------------------------------------
	//
	wxAuiNotebook * IRMMainFrame::createNotebook()
	{
		wxSize client_size = GetClientSize();
		wxAuiNotebook* noteBook = new wxAuiNotebook(this, -1, wxPoint(client_size.x, client_size.y), wxSize(800,425), m_notebook_style);
		wxBitmap page_bmp = wxArtProvider::GetBitmap(wxART_NORMAL_FILE, wxART_OTHER, wxSize(16,16));
		m_mgr.AddPane(noteBook,wxAuiPaneInfo().Name(wxT("editor_content")).Caption(wxT("Viewers")).MinimizeButton(true).MaximizeButton(true).CloseButton(false).Bottom().MinSize(wxSize(800,410)).MaxSize(wxSize(800,410)));
		return noteBook;
	}

	void IRMMainFrame::createGimmick()
	{
		if (_gimmickPanel==NULL)
		{
			//int min_image_type = GIMMICK_2D_IMAGE_SELECTION;
			//int max_image_type = GIMMICK_3D_IMAGE_SELECTION;
			//int output_dim = NATIVE;
			int threads = 1;

			_gimmickPanel = new creaImageIO::WxGimmickPanel(this,
															-1,
															wxDefaultPosition,
															wxSize(800,425),
															"creaIRM_Descriptor.dscp",
															"creatisIRM DB",
															threads);
			//_gimmickPanel->ConnectSendImageObserver( boost::bind( &IRMMainFrame::onSelectedImage  , this, _1 ) );
			m_mgr.AddPane(_gimmickPanel,wxAuiPaneInfo().Name(wxT("editor_content")).Caption(wxT("Gimmick")).MinimizeButton(true).MaximizeButton(true).CloseButton(false).Top().MinSize(wxSize(800,410)).MaxSize(wxSize(800,410)));
			m_mgr.Update();
		}
	}

	//------------------------------------------------------------------------------------------------------------
	// Creation and initialization of Panels
	//------------------------------------------------------------------------------------------------------------
	bool IRMMainFrame::configurePanels(IRMViewPanel * pannView)
	{
		bool configured = pannView!=NULL;
		configured &= pannView!=NULL;		

		if( pannView )
		{
			int winNum = (int)_notebook->GetPageCount()+1;
			std::ostringstream name;
			name << "       Viewer " << winNum << "       ";
			_notebook->AddPage( pannView, crea::std2wx(name.str()), true );		
			m_mgr.Update();
		}
 
		SetMinSize(wxSize(800,410));
		m_mgr.Update();
		return configured;
	}

	bool IRMMainFrame::configurePanelCreaContours(wxContourMainFrame* panelCreaContours)
	{
		bool configured = panelCreaContours!=NULL;
		configured &= panelCreaContours!=NULL;

		if( configured )
		{
			_notebook->AddPage( panelCreaContours, wxT("       CreaContours       "), true );
			m_mgr.Update();
			_contourID=(int)_notebook->GetSelection();
		}
		return configured;
	}

	bool IRMMainFrame::configurePanelSuperposition(wxMaracasSuperposition* panelSuperposition)
	{
		bool configured = panelSuperposition!=NULL;
		configured &= panelSuperposition!=NULL;

		if( configured )
		{
			_notebook->AddPage( panelSuperposition, wxT("       Superposition Tool       "), true );
			m_mgr.Update();
			_superpositionID=(int)_notebook->GetSelection();
		}
		return configured;
	}

	//------------------------------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------------------------------

	void IRMMainFrame::onCreateViewer()
	{
		if (_gimmickPanel!=NULL)
		{
			int output_dim = NATIVE;  // C'est ce qui precise a GetSelectedImages() sous quelle forme on veut les images
			std::vector<vtkImageData*> images;
			_gimmickPanel->GetSelectedImages(images, output_dim);
			//images[0]->PrintSelf(std::cout, vtkIndent(2));

			int size = images.size();

			if (size>0 && size<2)
			{
				int num;
				for (num=0; num<size; num++)
				{
					IRMViewPanel * viewPanel = new IRMViewPanel( images[num], _notebook );
					_appCore->addImage(images[num]);
					_viewers.push_back(viewPanel);
					configurePanels(viewPanel);
					_pannew->updateViewersInfo();
				}
			}

			else if (size>1)
			{
				wxMessageBox(_T("There is more than one image selected"), _T("Error"), wxICON_ERROR | wxOK);
			}
		}
	}

	void IRMMainFrame::onCreateMIPViewer()  /// \TODO add *actually* a MIP Viewer // JPR
	{
		if (_gimmickPanel!=NULL)
		{
			int output_dim = NATIVE; // C'est ce qui precise a GetSelectedImages() sous quelle forme on veut les images
			std::vector<vtkImageData*> images;
			_gimmickPanel->GetSelectedImages(images, output_dim);
			int size = images.size();

			if (size>0 && size<2)
			{
				int num;
				for (num=0; num<size; num++)
				{
					IRMViewPanel * viewPanel = new IRMViewPanel( images[num], _notebook );
					_appCore->addImage(images[num]);
					_viewers.push_back(viewPanel);
					configurePanels(viewPanel);
					_pannew->updateViewersInfo();
				}
			}

			else if (size>1)
			{
				wxMessageBox(_T("There is more than one image selected"), _T("Error"), wxICON_ERROR | wxOK);
			}
		}
	}

	void IRMMainFrame::onContourCreate(int selected)
	{
		if (_contourID==-1 && selected>=0)
		{
			vtkImageData* img = _appCore->getSelectedImage(selected);
			std::vector<vtkImageData*> selectedImages;
			selectedImages.push_back(img);
			_creaContours = wxContourMainFrame::getInstance(_notebook, wxID_ANY, wxString(_T("")), wxPoint(100,50), wxSize(800, 600), selectedImages, wxDEFAULT_FRAME_STYLE | wxSUNKEN_BORDER, _datadir  );
			configurePanelCreaContours(_creaContours);

			_pannew->hideWindow();
		}
	}

	void IRMMainFrame::onSuperpositionPanelCreate(int img1, int img2)
	{
		if (( img1!=-1 && img2!=-1 ) && img1 != img2)
		{
			vtkImageData * image1 = _appCore->getSelectedImage(img1);
			vtkImageData * image2 = _appCore->getSelectedImage(img2);
			std::vector<vtkImageData*> selectedImages;
			selectedImages.push_back(image1);
			selectedImages.push_back(image2);
			_superpositionPanel = wxMaracasSuperposition::getInstance(_notebook, selectedImages  );
			configurePanelSuperposition(_superpositionPanel);
		}
		else if (img1==img2)
		{
			wxMessageBox(_T("You have selected the same image to superpose"), _T("Error"), wxICON_ERROR | wxOK);
		}
	}

	void IRMMainFrame::onSelectedImage(bool t)
	{
		int output_dim = NATIVE;
		std::vector<vtkImageData*> images;
		_gimmickPanel->GetSelectedImages(images, output_dim);
		std::cout<<"Tamanho: " << images.size() << std::endl;

		if (images.size()>0)
		{
			//_theViewPanel1->ChangeImage(images[]);
			//_theViewPanel1->RefreshInterface();
		}
	}

	void IRMMainFrame::createADC(int inputImage, std::string path, int min, int max, bool addToDB)
	{
		/// \TODO: call Solene's Code to create an ADC Cartography
		_appCore->createADC(inputImage, path, min, max);	

		if (_gimmickPanel!=NULL && addToDB && path.compare("")!=0)
		{
			_gimmickPanel->AddImagesToDB(path);
		}	
	}

	void IRMMainFrame::createTTP( /*std::vector<std::string> inputImageS,*/  std::string path, int min, int max, std::string ficEntreeArt,
			double res_temp,
			int nbbaseline,
			int delay,
			double eps,
			double TE,
			double thresnoise,
			int retard_AIF,
			int retard_perf,
			bool verbose, 
			bool cartoMax, 
			bool cartoSVD, 
			bool cartoARMA,
			bool aff,
			bool enreg,
			bool addToDB,
			int firstSliceToDeal)  // JPR
	{
		if (_gimmickPanel==NULL)
		{
			std::cout << "No pannel ?!? " << std::endl;
			return;
		}

		int output_dim = NATIVE;  // a revoir !
		std::vector<vtkImageData*> images;
		//_gimmickPanel->GetSelectedImages(images, output_dim);
		_gimmickPanel->GetSelectedImagesInVector(images, output_dim);

		// call Solene's Code to create a TTP Cartography

		_appCore->createTTP(images, path, min, max, ficEntreeArt,
			 res_temp, 
			 nbbaseline, 
			 delay,
			 eps, 
			 TE, 
			 thresnoise, 
			 retard_AIF, 
			 retard_perf,
			 verbose, 
			 cartoMax,
			 cartoSVD, 
			 cartoARMA,
			 aff,
			 enreg,
			 firstSliceToDeal
			);

		if (_gimmickPanel!=NULL && addToDB && path.compare("")!=0)
		{
			_gimmickPanel->AddImagesToDB(path);
		}	
	}

// ==>
// ==>	// Probabely useless, since 'createTTP' creates, on user demand, cartoMax and/or cartoSVD and/or cartoARMA
// ==>
	void IRMMainFrame::createPeak(int inputImage, std::string path, int min, int max, bool addToDB)
	{
		/// \TODO: call Solene's Code to create a Peak Cartography
		_appCore->createPeak(inputImage, path, min, max);

		if (_gimmickPanel!=NULL && addToDB && path.compare("")!=0)
		{
			_gimmickPanel->AddImagesToDB(path);
		}	
	}

	//------------------------------------------------------------------------------------------------------------
	// Get/Set Methods
	//------------------------------------------------------------------------------------------------------------

	int IRMMainFrame::getDimZ(int selected)
	{
		int dimZ = 1;
		if (selected!=-1)
		{
			std::cout << "Selected: " << selected;
			dimZ = 	_viewers[selected]->GetImageDataSizeZ();
			std::cout << "     Slices: " << dimZ << std::endl;
		}
		return dimZ;
	}

	int IRMMainFrame::getNumberOfViewers()
	{
		return _viewers.size();
	}

	int IRMMainFrame::getSelectedView()
	{
		return _notebook->GetSelection();
	}

	void IRMMainFrame::changeViewer(int sel)
	{
		_notebook->SetSelection(sel);
	}

	//------------------------------------------------------------------------------------------------------------
	// Events Handlers
	//------------------------------------------------------------------------------------------------------------

	void IRMMainFrame::onPageClose(wxAuiNotebookEvent& event)
	{
		int selected = _notebook->GetSelection();
		
		if (selected==_contourID)
		{
			_notebook->DeletePage(selected);
			m_mgr.Update();

			//wxContourMainFrame::resetInstance();
			_creaContours=NULL;
			_contourID=-1;
			_pannew->showWindow();
		}

		else if (selected==_superpositionID)
		{
			_notebook->DeletePage(selected);
			m_mgr.Update();

			_superpositionPanel=NULL;
			_superpositionID=-1;
		}

		else if (selected!=_contourID)
		{
			// erase the selected element
			_appCore->removeImage(selected);
			_viewers.erase (_viewers.begin()+(selected));
			_notebook->DeletePage(selected);

			for (int i=selected; i<(int)_notebook->GetPageCount(); i++)
			{
				if ((i+1)!=_contourID)
				{
					std::ostringstream name;
					name << "       Viewer " << i+1 << "       ";
					_notebook->SetPageText(i, crea::std2wx(name.str()));
				}
			}
			
			if (_contourID!=-1)
			{
				_contourID = _contourID-1;
			}

			//std::cout<<"NumViewer: "<<_viewers.size()<<std::endl;
			_pannew->updateViewersInfo();
		}
		event.Veto();
	}

	void IRMMainFrame::onPageChanged(wxAuiNotebookEvent& event)
	{
		if (_pannew!=NULL && _notebook->GetSelection()!=_contourID && _notebook->GetSelection()!=-1)
		{		
			_pannew->updateViewersInfo();
		}
	}

	void IRMMainFrame::onPaneClose(wxAuiManagerEvent& evt)
	{
		std::cout  << "Closing panel '"<<evt.pane->name<<"'"<<std::endl;
		// TO DO : Uncheck 
		//    OnMenuWindowsCheck( wxCommandEvent& event, wxWindow* w);
	}

	BEGIN_EVENT_TABLE(IRMMainFrame, wxPanel)
		EVT_AUINOTEBOOK_PAGE_CLOSE(wxID_ANY, IRMMainFrame::onPageClose)
		EVT_AUI_PANE_CLOSE(IRMMainFrame::onPaneClose)
		EVT_AUINOTEBOOK_PAGE_CHANGED(wxID_ANY, IRMMainFrame::onPageChanged)
	END_EVENT_TABLE()
