#ifndef __IRMMainFrame_h_INCLUDED_H__
#define __IRMMainFrame_h_INCLUDED_H__


// -----------------------------------------------------------------------------------------------------------
// WX headers inclusion.
// For compilers that support precompilation, includes <wx/wx.h>.
// -----------------------------------------------------------------------------------------------------------

#include <string>
#include <wx/wxprec.h>
#ifdef __BORLANDC__
#pragma hdrstop
#endif
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

//------------------------------------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------------------------------------
#include "wx/aui/aui.h"

#include <creaImageIOWxGimmickPanel.h>
#include <creaImageIOSystem.h>
#include "wxContourMainFrame.h"
#include "wxMaracasSuperposition.h"
#include "buttonsMainPanel.h"
#include "IRMViewPanel.h"
#include "IRMCore.h"
#include "vtkMetaImageReader.h"
#include "vtkImageData.h"

//------------------------------------------------------------------------------------------------------------
// Events declaration
//------------------------------------------------------------------------------------------------------------

//BEGIN_DECLARE_EVENT_TYPES()
//END_DECLARE_EVENT_TYPES()

/** file IRMMainFrame.h */

//------------------------------------------------------------------------------------------------------------
// Class definition
//------------------------------------------------------------------------------------------------------------

class buttonsMainPanel;
class IRMMainFrame : public wxPanel 
{
	public:

	//------------------------------------------------------------------------------------------------------------
	// Constructors & Destructors
	//------------------------------------------------------------------------------------------------------------

	IRMMainFrame(wxWindow* parent, wxWindowID id,const wxString& title,const wxPoint& pos,const wxSize& size, long style  = wxDEFAULT_FRAME_STYLE | wxSUNKEN_BORDER,std::string datadir="<void>");

	~IRMMainFrame();

	//------------------------------------------------------------------------------------------------------------
	// Creational and initialization methods
	//------------------------------------------------------------------------------------------------------------
	bool configurePanels(IRMViewPanel * pannView);
	bool configurePanelCreaContours(wxContourMainFrame* panelCreaContours);
	bool configurePanelSuperposition(wxMaracasSuperposition* panelSuperposition);

	wxAuiNotebook * createNotebook();

	void createGimmick();

	//------------------------------------------------------------------------------------------------------------
	//  Attributes getters and setters
	//------------------------------------------------------------------------------------------------------------
	static IRMMainFrame* getInstance(wxWindow* parent, wxWindowID id,const wxString& title,const wxPoint& pos,const wxSize& size, long style  = wxDEFAULT_FRAME_STYLE | wxSUNKEN_BORDER, std::string datadir="<void>");
	static IRMMainFrame* getInstance();
	//------------------------------------------------------------------------------------------------------------
	//  Functional methods
	//------------------------------------------------------------------------------------------------------------

	void onCreateViewer();
	void onCreateMIPViewer();	
	void onContourCreate(int selected);
	void onSuperpositionPanelCreate(int img1, int img2);
	void onPageClose(wxAuiNotebookEvent& event);
	void onPaneClose(wxAuiManagerEvent& evt);
	void onPageChanged(wxAuiNotebookEvent& event);

	void onSelectedImage (bool t);

	// Some more parameters needed ! // JPR
	void createADC (int inputImage,  std::string path, int min, int max, bool addToDB);
	void createPeak(int inputImage,  std::string path, int min, int max, bool addToDB);

	void createTTP (std::string path, int min, int max, std::string ficEntreeArt,
			double res_temp, int nbbaseline, int delay, double eps, double TE, double thresnoise, int retard_AIF, int retard_perf,
			bool verbose, 
			bool cartoMax, 
			bool cartoSVD, 
			bool cartoARMA, 
			bool aff,
			bool enreg,
			bool addToDB,
			int firstSliceToDeal); // JPR

	//------------------------------------------------------------------------------------------------------------
	// Get/Set Methods
	//------------------------------------------------------------------------------------------------------------

	int getDimZ(int selected);
	int getNumberOfViewers();
	int getSelectedView();
	void changeViewer(int sel);

	//------------------------------------------------------------------------------------------------------------
	// Attributtes
	//------------------------------------------------------------------------------------------------------------
	static IRMMainFrame* instance;

	IRMCore * _appCore;
	wxAuiManager m_mgr;
	wxAuiNotebook * _notebook;
	wxContourMainFrame* _creaContours;
	wxMaracasSuperposition * _superpositionPanel;
	creaImageIO::WxGimmickPanel * _gimmickPanel;
	std::vector<IRMViewPanel*> _viewers;
	buttonsMainPanel * _pannew;

	int _contourID;
	int _superpositionID;
	std::string	_datadir;

	long m_notebook_style;
	long m_notebook_theme;

	DECLARE_EVENT_TABLE()
};

#endif 
